from __future__ import annotations

from dataclasses import dataclass
from enum import Enum, auto
import logging
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from collections.abc import Sequence

    from .primitive import XspecPrimitive, XspecConvPrimitive

try:
    from ._compiled import *
    __version__ = _compiled.__version__
    __INITIALIZED__ = True
except ImportError as ie:
    # Allow the actual error message to be reported if the user
    # has tweaked the log level, for instance with:
    #
    #   import logging; logging.basicConfig(level=logging.DEBUG)
    #
    logging.getLogger(__name__).warn("Unable to import compiled Xspec models")
    logging.getLogger(__name__).info(str(ie))

    __version__ = "none"
    __INITIALIZED__ = False

_compiled._init()


class ModelType(Enum):
    """The various Xspec model types."""

    Add = auto()
    Mul = auto()
    Con = auto()


class LanguageStyle(Enum):
    """The various ways to define and call Xspec models."""

    CppStyle8 = auto()
    CStyle8 = auto()
    F77Style4 = auto()
    F77Style8 = auto()


class ParamType(Enum):
    """The Xspec parameter type."""

    Default = auto()
    Switch = auto()
    Scale = auto()
    Periodic = auto()


@dataclass
class XspecParameter:
    """An Xspec parameter."""

    paramtype: ParamType
    name: str
    default: float
    units: str | None = None
    frozen: bool = False
    # Would it be better to just have limits = [hardmin, softmni, softmax, hardmax]?
    softmin: float | None = None
    softmax: float | None = None
    hardmin: float | None = None
    hardmax: float | None = None
    delta: float | None = None


@dataclass
class XspecModel:
    """An Xspec model."""

    modeltype: ModelType
    name: str
    funcname: str
    language: LanguageStyle
    elo: float
    ehi: float
    parameters: Sequence[XspecParameter]
    use_errors: bool = False
    can_cache: bool = True


_info = {
    'agauss': XspecModel(modeltype=ModelType.Add, name='agauss', funcname='agauss', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='LineE', default=10.0, units='A', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sigma', default=1.0, units='A', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01)]),
    'agnsed': XspecModel(modeltype=ModelType.Add, name='agnsed', funcname='agnsed', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='mass', default=10000000.0, units='solar', frozen=True, softmin=1.0, softmax=10000000000.0, hardmin=1.0, hardmax=10000000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='dist', default=100.0, units='Mpc', frozen=True, softmin=0.01, softmax=1000000000.0, hardmin=0.01, hardmax=1000000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='logmdot', default=-1.0, softmin=-10.0, softmax=2.0, hardmin=-10.0, hardmax=2.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='astar', default=0.0, frozen=True, softmin=-1.0, softmax=0.998, hardmin=-1.0, hardmax=0.998, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='cosi', default=0.5, frozen=True, softmin=0.05, softmax=1.0, hardmin=0.05, hardmax=1.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='kTe_hot', default=100.0, units='keV(_pl)', frozen=True, softmin=10.0, softmax=300.0, hardmin=10.0, hardmax=300.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='kTe_warm', default=0.2, units='keV(_sc)', softmin=0.1, softmax=0.5, hardmin=0.1, hardmax=0.5, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Gamma_hot', default=1.7, units='(_calc)', softmin=1.3, softmax=3.0, hardmin=1.3, hardmax=3.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Gamma_warm', default=2.7, units='(_disk)', softmin=2.0, softmax=5.0, hardmin=2.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='R_hot', default=10.0, units='Rg', softmin=6.0, softmax=500.0, hardmin=6.0, hardmax=500.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='R_warm', default=20.0, units='Rg', softmin=6.0, softmax=500.0, hardmin=6.0, hardmax=500.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='logrout', default=-1.0, units='(_selfg)', frozen=True, softmin=-3.0, softmax=7.0, hardmin=-3.0, hardmax=7.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Htmax', default=10.0, units='Rg', frozen=True, softmin=6.0, softmax=10.0, hardmin=6.0, hardmax=10.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='reprocess', default=1.0, units='0off/1on', frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=1.0)]),
    'agnslim': XspecModel(modeltype=ModelType.Add, name='agnslim', funcname='agnslim', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='mass', default=10000000.0, units='solar', frozen=True, softmin=1.0, softmax=10000000000.0, hardmin=1.0, hardmax=10000000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='dist', default=100.0, units='Mpc', frozen=True, softmin=0.01, softmax=1000000000.0, hardmin=0.01, hardmax=1000000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='logmdot', default=1.0, softmin=-10.0, softmax=3.0, hardmin=-10.0, hardmax=3.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='astar', default=0.0, frozen=True, softmin=0.0, softmax=0.998, hardmin=0.0, hardmax=0.998, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='cosi', default=0.5, frozen=True, softmin=0.05, softmax=1.0, hardmin=0.05, hardmax=1.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='kTe_hot', default=100.0, units='keV(-pl)', frozen=True, softmin=10.0, softmax=300.0, hardmin=10.0, hardmax=300.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='kTe_warm', default=0.2, units='keV(-sc)', softmin=0.1, softmax=0.5, hardmin=0.1, hardmax=0.5, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Gamma_hot', default=2.4, softmin=1.3, softmax=3.0, hardmin=1.3, hardmax=3.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Gamma_warm', default=3.0, units='(-disk)', softmin=2.0, softmax=5.0, hardmin=2.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='R_hot', default=10.0, units='Rg', softmin=2.0, softmax=500.0, hardmin=2.0, hardmax=500.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='R_warm', default=20.0, units='Rg', softmin=2.0, softmax=500.0, hardmin=2.0, hardmax=500.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='logrout', default=-1.0, units='(-selfg)', frozen=True, softmin=-3.0, softmax=7.0, hardmin=-3.0, hardmax=7.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='rin', default=-1.0, frozen=True, softmin=-1.0, softmax=100.0, hardmin=-1.0, hardmax=100.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=1.0)]),
    'apec': XspecModel(modeltype=ModelType.Add, name='apec', funcname='apec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.008, softmax=64.0, hardmin=0.008, hardmax=64.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'bapec': XspecModel(modeltype=ModelType.Add, name='bapec', funcname='bapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.008, softmax=64.0, hardmin=0.008, hardmax=64.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0)]),
    'btapec': XspecModel(modeltype=ModelType.Add, name='btapec', funcname='btapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.008, softmax=64.0, hardmin=0.008, hardmax=64.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kTi', default=1.0, units='keV', softmin=0.008, softmax=64.0, hardmin=0.008, hardmax=64.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0)]),
    'bbody': XspecModel(modeltype=ModelType.Add, name='bbody', funcname='xsblbd', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=3.0, units='keV', softmin=0.01, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.01)]),
    'bbodyrad': XspecModel(modeltype=ModelType.Add, name='bbodyrad', funcname='xsbbrd', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=3.0, units='keV', softmin=0.001, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.01)]),
    'bexrav': XspecModel(modeltype=ModelType.Add, name='bexrav', funcname='xsbexrav', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Gamma1', default=2.0, softmin=-9.0, softmax=9.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='breakE', default=10.0, units='keV', softmin=0.1, softmax=1000.0, hardmin=0.1, hardmax=1000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Gamma2', default=2.0, softmin=-9.0, softmax=9.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='foldE', default=100.0, units='keV', softmin=1.0, softmax=1000000.0, hardmin=1.0, hardmax=1000000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=0.0, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.45, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'bexriv': XspecModel(modeltype=ModelType.Add, name='bexriv', funcname='xsbexriv', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Gamma1', default=2.0, softmin=-9.0, softmax=9.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='breakE', default=10.0, units='keV', softmin=0.1, softmax=1000.0, hardmin=0.1, hardmax=1000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Gamma2', default=2.0, softmin=-9.0, softmax=9.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='foldE', default=100.0, units='keV', softmin=1.0, softmax=1000000.0, hardmin=1.0, hardmax=1000000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=0.0, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.45, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='T_disk', default=30000.0, units='K', frozen=True, softmin=10000.0, softmax=1000000.0, hardmin=10000.0, hardmax=1000000.0, delta=1000.0), XspecParameter(paramtype=ParamType.Default, name='xi', default=1.0, units='erg cm/s', softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=5000.0, delta=0.1)]),
    'bknpower': XspecModel(modeltype=ModelType.Add, name='bknpower', funcname='brokenPowerLaw', language=LanguageStyle.CppStyle8, elo=-1e+20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndx1', default=1.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='BreakE', default=5.0, units='keV', softmin=0.01, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='PhoIndx2', default=2.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01)]),
    'bkn2pow': XspecModel(modeltype=ModelType.Add, name='bkn2pow', funcname='broken2PowerLaw', language=LanguageStyle.CppStyle8, elo=-1e+20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndx1', default=1.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='BreakE1', default=5.0, units='keV', softmin=0.01, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='PhoIndx2', default=2.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='BreakE2', default=10.0, units='keV', softmin=0.01, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='PhoIndx3', default=3.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01)]),
    'bmc': XspecModel(modeltype=ModelType.Add, name='bmc', funcname='xsbmc', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.01, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='alpha', default=1.0, softmin=0.01, softmax=4.0, hardmin=0.0001, hardmax=6.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='log_A', default=0.0, softmin=-6.0, softmax=6.0, hardmin=-8.0, hardmax=8.0, delta=0.01)]),
    'bremss': XspecModel(modeltype=ModelType.Add, name='bremss', funcname='xsbrms', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=7.0, units='keV', softmin=0.0001, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.05)]),
    'brnei': XspecModel(modeltype=ModelType.Add, name='brnei', funcname='brnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=0.5, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_init', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0)]),
    'bvapec': XspecModel(modeltype=ModelType.Add, name='bvapec', funcname='bvapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0)]),
    'bvrnei': XspecModel(modeltype=ModelType.Add, name='bvrnei', funcname='bvrnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=0.5, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_init', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0)]),
    'bvtapec': XspecModel(modeltype=ModelType.Add, name='bvtapec', funcname='bvtapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kTi', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0)]),
    'bvvapec': XspecModel(modeltype=ModelType.Add, name='bvvapec', funcname='bvvapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0)]),
    'bvvrnei': XspecModel(modeltype=ModelType.Add, name='bvvrnei', funcname='bvvrnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=0.5, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_init', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0)]),
    'bvvtapec': XspecModel(modeltype=ModelType.Add, name='bvvtapec', funcname='bvvtapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kTi', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0)]),
    'c6mekl': XspecModel(modeltype=ModelType.Add, name='c6mekl', funcname='c6mekl', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='CPcoef1', default=1.0, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='CPcoef2', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef3', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef4', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef5', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef6', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abundanc', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True)]),
    'c6pmekl': XspecModel(modeltype=ModelType.Add, name='c6pmekl', funcname='c6pmekl', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='CPcoef1', default=1.0, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='CPcoef2', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef3', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef4', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef5', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef6', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abundanc', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True)]),
    'c6pvmkl': XspecModel(modeltype=ModelType.Add, name='c6pvmkl', funcname='c6pvmkl', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='CPcoef1', default=1.0, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='CPcoef2', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef3', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef4', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef5', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef6', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True)]),
    'c6vmekl': XspecModel(modeltype=ModelType.Add, name='c6vmekl', funcname='c6vmekl', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='CPcoef1', default=1.0, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='CPcoef2', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef3', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef4', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef5', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CPcoef6', default=0.5, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True)]),
    'carbatm': XspecModel(modeltype=ModelType.Add, name='carbatm', funcname='carbatm', language=LanguageStyle.CppStyle8, elo=0.0, ehi=20.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='T', default=2.0, units='MK', softmin=1.0, softmax=4.0, hardmin=1.0, hardmax=4.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='NSmass', default=1.4, units='Msun', softmin=0.6, softmax=2.8, hardmin=0.6, hardmax=2.8, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='NSrad', default=10.0, units='km', softmin=6.0, softmax=23.0, hardmin=6.0, hardmax=23.0, delta=0.001)]),
    'cemekl': XspecModel(modeltype=ModelType.Add, name='cemekl', funcname='cemekl', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='alpha', default=1.0, frozen=True, softmin=0.01, softmax=10.0, hardmin=0.01, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tmax', default=1.0, units='keV', softmin=0.02725, softmax=100.0, hardmin=0.02725, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abundanc', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=-1.0)]),
    'cevmkl': XspecModel(modeltype=ModelType.Add, name='cevmkl', funcname='cemVMekal', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='alpha', default=1.0, frozen=True, softmin=0.01, softmax=10.0, hardmin=0.01, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tmax', default=1.0, units='keV', softmin=0.02725, softmax=100.0, hardmin=0.02725, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=-1.0)]),
    'cflow': XspecModel(modeltype=ModelType.Add, name='cflow', funcname='xscflw', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='slope', default=0.0, softmin=-5.0, softmax=5.0, hardmin=-5.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='lowT', default=0.1, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='highT', default=4.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.1, frozen=True, softmin=1e-10, softmax=10.0, hardmin=1e-10, hardmax=10.0, delta=0.01)]),
    'compbb': XspecModel(modeltype=ModelType.Add, name='compbb', funcname='compbb', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.01, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kTe', default=50.0, units='keV', frozen=True, softmin=1.0, softmax=200.0, hardmin=1.0, hardmax=200.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='tau', default=0.1, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001)]),
    'compmag': XspecModel(modeltype=ModelType.Add, name='compmag', funcname='xscompmag', language=LanguageStyle.CStyle8, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kTbb', default=1.0, units='keV', softmin=0.2, softmax=10.0, hardmin=0.2, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='kTe', default=5.0, units='keV', softmin=0.2, softmax=2000.0, hardmin=0.2, hardmax=2000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='tau', default=0.5, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='eta', default=0.5, softmin=0.01, softmax=1.0, hardmin=0.01, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='beta0', default=0.57, softmin=0.0001, softmax=1.0, hardmin=0.0001, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='r0', default=0.25, softmin=0.0001, softmax=100.0, hardmin=0.0001, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='A', default=0.001, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='betaflag', default=1.0, frozen=True, softmin=0.0, softmax=2.0, hardmin=0.0, hardmax=2.0, delta=0.01)]),
    'compLS': XspecModel(modeltype=ModelType.Add, name='compLS', funcname='compls', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=2.0, units='keV', softmin=0.01, softmax=10.0, hardmin=0.001, hardmax=20.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='tau', default=10.0, softmin=0.001, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.0001)]),
    'compPS': XspecModel(modeltype=ModelType.Add, name='compPS', funcname='xscompps', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kTe', default=100.0, units='keV', softmin=20.0, softmax=100000.0, hardmin=20.0, hardmax=100000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='EleIndex', default=2.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Gmin', default=-1.0, frozen=True, softmin=-1.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Gmax', default=1000.0, frozen=True, softmin=10.0, softmax=10000.0, hardmin=10.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='kTbb', default=0.1, units='keV', frozen=True, softmin=0.001, softmax=10.0, hardmin=0.001, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='tau_y', default=1.0, softmin=0.05, softmax=3.0, hardmin=0.05, hardmax=3.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='geom', default=0.0, frozen=True, softmin=-5.0, softmax=4.0, hardmin=-5.0, hardmax=4.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='HovR_cyl', default=1.0, frozen=True, softmin=0.5, softmax=2.0, hardmin=0.5, hardmax=2.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.5, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='cov_frac', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=0.0, frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Fe_ab_re', default=1.0, frozen=True, softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Me_ab', default=1.0, frozen=True, softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='xi', default=0.0, frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=100000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Tdisk', default=1000000.0, units='K', frozen=True, softmin=10000.0, softmax=1000000.0, hardmin=10000.0, hardmax=1000000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='Betor10', default=-10.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rin', default=10.0, units='Rs', frozen=True, softmin=6.001, softmax=1000.0, hardmin=6.001, hardmax=10000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout', default=1000.0, units='Rs', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'compST': XspecModel(modeltype=ModelType.Add, name='compST', funcname='compst', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=2.0, units='keV', softmin=0.01, softmax=100.0, hardmin=0.001, hardmax=100.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='tau', default=10.0, softmin=0.001, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.0001)]),
    'comptb': XspecModel(modeltype=ModelType.Add, name='comptb', funcname='xscomptb', language=LanguageStyle.CStyle8, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kTs', default=1.0, units='keV', softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='gamma', default=3.0, frozen=True, softmin=1.0, softmax=10.0, hardmin=1.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='alpha', default=2.0, softmin=0.0, softmax=400.0, hardmin=0.0, hardmax=400.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='delta', default=20.0, softmin=0.0, softmax=200.0, hardmin=0.0, hardmax=200.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kTe', default=5.0, units='keV', softmin=0.2, softmax=2000.0, hardmin=0.2, hardmax=2000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_A', default=0.0, softmin=-8.0, softmax=8.0, hardmin=-8.0, hardmax=8.0, delta=0.01)]),
    'compth': XspecModel(modeltype=ModelType.Add, name='compth', funcname='xscompth', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='theta', default=1.0, units='keV', softmin=1e-06, softmax=1000000.0, hardmin=1e-06, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='showbb', default=1.0, frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='kT_bb', default=200.0, units='eV', frozen=True, softmin=1.0, softmax=400000.0, hardmin=1.0, hardmax=400000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='RefOn', default=-1.0, frozen=True, softmin=-2.0, softmax=2.0, hardmin=-2.0, hardmax=2.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='tau_p', default=0.1, frozen=True, softmin=0.0001, softmax=10.0, hardmin=0.0001, hardmax=10.0, delta=0.0001), XspecParameter(paramtype=ParamType.Default, name='radius', default=10000000.0, units='cm', frozen=True, softmin=100000.0, softmax=1e+16, hardmin=100000.0, hardmax=1e+16, delta=100000.0), XspecParameter(paramtype=ParamType.Default, name='g_min', default=1.3, frozen=True, softmin=1.2, softmax=1000.0, hardmin=1.2, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='g_max', default=1000.0, frozen=True, softmin=5.0, softmax=10000.0, hardmin=5.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='G_inj', default=2.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='pairinj', default=0.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.5, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Refl', default=1.0, frozen=True, softmin=0.0, softmax=2.0, hardmin=0.0, hardmax=2.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ab_met', default=1.0, frozen=True, softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='T_disk', default=1000000.0, units='K', frozen=True, softmin=10000.0, softmax=1000000.0, hardmin=10000.0, hardmax=1000000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='xi', default=0.0, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=5000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='Beta', default=-10.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rin', default=10.0, units='M', frozen=True, softmin=6.001, softmax=1000.0, hardmin=6.001, hardmax=10000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout', default=1000.0, units='M', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=4.0, hardmin=0.0, hardmax=4.0, delta=0.01)]),
    'compTT': XspecModel(modeltype=ModelType.Add, name='compTT', funcname='xstitg', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='T0', default=0.1, units='keV', softmin=0.01, softmax=100.0, hardmin=0.001, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT', default=50.0, units='keV', softmin=2.0, softmax=500.0, hardmin=2.0, hardmax=500.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='taup', default=1.0, softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=200.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='approx', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=200.0, delta=0.1)]),
    'cph': XspecModel(modeltype=ModelType.Add, name='cph', funcname='cph', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='peakT', default=2.2, units='keV', softmin=0.1, softmax=100.0, hardmin=0.1, hardmax=100.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Abund', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=0.0, softmax=50.0, hardmin=0.0, hardmax=50.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True)]),
    'cplinear': XspecModel(modeltype=ModelType.Add, name='cplinear', funcname='cplinear', language=LanguageStyle.CppStyle8, elo=0.0, ehi=10.0, parameters=[XspecParameter(paramtype=ParamType.Scale, name='energy00', default=0.5, units='keV', frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=-1.0), XspecParameter(paramtype=ParamType.Scale, name='energy01', default=1.0, units='keV', frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=-1.0), XspecParameter(paramtype=ParamType.Scale, name='energy02', default=1.5, units='keV', frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=-1.0), XspecParameter(paramtype=ParamType.Scale, name='energy03', default=2.0, units='keV', frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=-1.0), XspecParameter(paramtype=ParamType.Scale, name='energy04', default=3.0, units='keV', frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=-1.0), XspecParameter(paramtype=ParamType.Scale, name='energy05', default=4.0, units='keV', frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=-1.0), XspecParameter(paramtype=ParamType.Scale, name='energy06', default=5.0, units='keV', frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=-1.0), XspecParameter(paramtype=ParamType.Scale, name='energy07', default=6.0, units='keV', frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=-1.0), XspecParameter(paramtype=ParamType.Scale, name='energy08', default=7.0, units='keV', frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=-1.0), XspecParameter(paramtype=ParamType.Scale, name='energy09', default=8.0, units='keV', frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=-1.0), XspecParameter(paramtype=ParamType.Default, name='log_rate00', default=0.0, frozen=True, softmin=-19.0, softmax=19.0, hardmin=-20.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_rate01', default=1.0, frozen=True, softmin=-19.0, softmax=19.0, hardmin=-20.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_rate02', default=0.0, frozen=True, softmin=-19.0, softmax=19.0, hardmin=-20.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_rate03', default=1.0, frozen=True, softmin=-19.0, softmax=19.0, hardmin=-20.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_rate04', default=0.0, frozen=True, softmin=-19.0, softmax=19.0, hardmin=-20.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_rate05', default=1.0, frozen=True, softmin=-19.0, softmax=19.0, hardmin=-20.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_rate06', default=0.0, frozen=True, softmin=-19.0, softmax=19.0, hardmin=-20.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_rate07', default=1.0, frozen=True, softmin=-19.0, softmax=19.0, hardmin=-20.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_rate08', default=0.0, frozen=True, softmin=-19.0, softmax=19.0, hardmin=-20.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_rate09', default=1.0, frozen=True, softmin=-19.0, softmax=19.0, hardmin=-20.0, hardmax=20.0, delta=0.1)]),
    'cutoffpl': XspecModel(modeltype=ModelType.Add, name='cutoffpl', funcname='cutoffPowerLaw', language=LanguageStyle.CppStyle8, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=1.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='HighECut', default=15.0, units='keV', softmin=1.0, softmax=500.0, hardmin=0.01, hardmax=500.0, delta=0.01)]),
    'disk': XspecModel(modeltype=ModelType.Add, name='disk', funcname='disk', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='accrate', default=1.0, softmin=0.001, softmax=9.0, hardmin=0.0001, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CenMass', default=1.4, units='Msun', frozen=True, softmin=0.4, softmax=10.0, hardmin=0.1, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rinn', default=1.03, frozen=True, softmin=1.01, softmax=1.03, hardmin=1.0, hardmax=1.04, delta=0.001)]),
    'diskir': XspecModel(modeltype=ModelType.Add, name='diskir', funcname='diskir', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT_disk', default=1.0, units='keV', softmin=0.01, softmax=5.0, hardmin=0.01, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Gamma', default=1.7, softmin=1.001, softmax=5.0, hardmin=1.001, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_e', default=100.0, units='keV', softmin=5.0, softmax=1000.0, hardmin=1.0, hardmax=1000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='LcovrLd', default=0.1, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='fin', default=0.1, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=1e-06), XspecParameter(paramtype=ParamType.Default, name='rirr', default=1.2, softmin=1.0001, softmax=10.0, hardmin=1.0001, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='fout', default=0.0001, softmin=0.0, softmax=0.1, hardmin=0.0, hardmax=0.1, delta=1e-06), XspecParameter(paramtype=ParamType.Default, name='logrout', default=5.0, softmin=3.0, softmax=7.0, hardmin=3.0, hardmax=7.0, delta=0.01)]),
    'diskbb': XspecModel(modeltype=ModelType.Add, name='diskbb', funcname='xsdskb', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Tin', default=1.0, units='keV', softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01)]),
    'diskline': XspecModel(modeltype=ModelType.Add, name='diskline', funcname='diskline', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='LineE', default=6.7, units='keV', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Betor10', default=-2.0, frozen=True, softmin=-10.0, softmax=20.0, hardmin=-10.0, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rin_M', default=10.0, frozen=True, softmin=6.0, softmax=1000.0, hardmin=6.0, hardmax=10000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout_M', default=1000.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=10000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Incl', default=30.0, units='deg', softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=0.05)]),
    'diskm': XspecModel(modeltype=ModelType.Add, name='diskm', funcname='diskm', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='accrate', default=1.0, softmin=0.001, softmax=9.0, hardmin=0.0001, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='NSmass', default=1.4, units='Msun', frozen=True, softmin=0.4, softmax=10.0, hardmin=0.1, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rinn', default=1.03, frozen=True, softmin=1.01, softmax=1.03, hardmin=1.0, hardmax=1.04, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='alpha', default=1.0, frozen=True, softmin=0.01, softmax=10.0, hardmin=0.001, hardmax=20.0, delta=0.001)]),
    'disko': XspecModel(modeltype=ModelType.Add, name='disko', funcname='disko', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='accrate', default=1.0, softmin=0.001, softmax=9.0, hardmin=0.0001, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='NSmass', default=1.4, units='Msun', frozen=True, softmin=0.4, softmax=10.0, hardmin=0.1, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rinn', default=1.03, frozen=True, softmin=1.01, softmax=1.03, hardmin=1.0, hardmax=1.04, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='alpha', default=1.0, frozen=True, softmin=0.01, softmax=10.0, hardmin=0.001, hardmax=20.0, delta=0.001)]),
    'diskpbb': XspecModel(modeltype=ModelType.Add, name='diskpbb', funcname='diskpbb', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Tin', default=1.0, units='keV', softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.02), XspecParameter(paramtype=ParamType.Default, name='p', default=0.75, softmin=0.5, softmax=1.0, hardmin=0.5, hardmax=1.0, delta=0.05)]),
    'diskpn': XspecModel(modeltype=ModelType.Add, name='diskpn', funcname='xsdiskpn', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='T_max', default=1.0, units='keV', softmin=0.001, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='R_in', default=6.0, units='R_g', softmin=6.0, softmax=1000.0, hardmin=6.0, hardmax=1000.0, delta=0.1)]),
    'eplogpar': XspecModel(modeltype=ModelType.Add, name='eplogpar', funcname='eplogpar', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Ep', default=0.1, units='keV', softmin=1e-06, softmax=100.0, hardmin=1e-10, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='beta', default=0.2, softmin=-4.0, softmax=4.0, hardmin=-4.0, hardmax=4.0, delta=0.01)]),
    'eqpair': XspecModel(modeltype=ModelType.Add, name='eqpair', funcname='xseqpair', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='l_hovl_s', default=1.0, softmin=1e-06, softmax=1000000.0, hardmin=1e-06, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='l_bb', default=100.0, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='kT_bb', default=200.0, units='eV', frozen=True, softmin=1.0, softmax=400000.0, hardmin=1.0, hardmax=400000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='l_ntol_h', default=0.5, softmin=0.0, softmax=0.9999, hardmin=0.0, hardmax=0.9999, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='tau_p', default=0.1, frozen=True, softmin=0.0001, softmax=10.0, hardmin=0.0001, hardmax=10.0, delta=0.0001), XspecParameter(paramtype=ParamType.Default, name='radius', default=10000000.0, units='cm', frozen=True, softmin=100000.0, softmax=1e+16, hardmin=100000.0, hardmax=1e+16, delta=100000.0), XspecParameter(paramtype=ParamType.Default, name='g_min', default=1.3, frozen=True, softmin=1.2, softmax=1000.0, hardmin=1.2, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='g_max', default=1000.0, frozen=True, softmin=5.0, softmax=10000.0, hardmin=5.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='G_inj', default=2.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='pairinj', default=0.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.5, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Refl', default=1.0, frozen=True, softmin=0.0, softmax=2.0, hardmin=0.0, hardmax=2.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ab_met', default=1.0, frozen=True, softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='T_disk', default=1000000.0, units='K', frozen=True, softmin=10000.0, softmax=1000000.0, hardmin=10000.0, hardmax=1000000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='xi', default=0.0, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=5000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='Beta', default=-10.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rin', default=10.0, units='M', frozen=True, softmin=6.001, softmax=1000.0, hardmin=6.001, hardmax=10000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout', default=1000.0, units='M', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=4.0, hardmin=0.0, hardmax=4.0, delta=0.01)]),
    'eqtherm': XspecModel(modeltype=ModelType.Add, name='eqtherm', funcname='xseqth', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='l_hovl_s', default=1.0, softmin=1e-06, softmax=1000000.0, hardmin=1e-06, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='l_bb', default=100.0, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='kT_bb', default=200.0, units='eV', frozen=True, softmin=1.0, softmax=400000.0, hardmin=1.0, hardmax=400000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='l_ntol_h', default=0.5, softmin=0.0, softmax=0.9999, hardmin=0.0, hardmax=0.9999, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='tau_p', default=0.1, frozen=True, softmin=0.0001, softmax=10.0, hardmin=0.0001, hardmax=10.0, delta=0.0001), XspecParameter(paramtype=ParamType.Default, name='radius', default=10000000.0, units='cm', frozen=True, softmin=100000.0, softmax=1e+16, hardmin=100000.0, hardmax=1e+16, delta=100000.0), XspecParameter(paramtype=ParamType.Default, name='g_min', default=1.3, frozen=True, softmin=1.2, softmax=1000.0, hardmin=1.2, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='g_max', default=1000.0, frozen=True, softmin=5.0, softmax=10000.0, hardmin=5.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='G_inj', default=2.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='pairinj', default=0.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.5, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Refl', default=1.0, frozen=True, softmin=0.0, softmax=2.0, hardmin=0.0, hardmax=2.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ab_met', default=1.0, frozen=True, softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='T_disk', default=1000000.0, units='K', frozen=True, softmin=10000.0, softmax=1000000.0, hardmin=10000.0, hardmax=1000000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='xi', default=0.0, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=5000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='Beta', default=-10.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rin', default=10.0, units='M', frozen=True, softmin=6.001, softmax=1000.0, hardmin=6.001, hardmax=10000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout', default=1000.0, units='M', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=4.0, hardmin=0.0, hardmax=4.0, delta=0.01)]),
    'equil': XspecModel(modeltype=ModelType.Add, name='equil', funcname='equil', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'expdec': XspecModel(modeltype=ModelType.Add, name='expdec', funcname='xsxpdec', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='factor', default=1.0, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01)]),
    'ezdiskbb': XspecModel(modeltype=ModelType.Add, name='ezdiskbb', funcname='ezdiskbb', language=LanguageStyle.F77Style4, elo=0.01, ehi=100.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='T_max', default=1.0, units='keV', softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=100.0, delta=0.01)]),
    'gaussian': XspecModel(modeltype=ModelType.Add, name='gaussian', funcname='gaussianLine', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='LineE', default=6.5, units='keV', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Sigma', default=0.1, units='keV', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.05)]),
    'gadem': XspecModel(modeltype=ModelType.Add, name='gadem', funcname='gaussDem', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Tmean', default=4.0, units='keV', frozen=True, softmin=0.01, softmax=10.0, hardmin=0.01, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tsigma', default=0.1, units='keV', softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abundanc', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=2, frozen=True)]),
    'gnei': XspecModel(modeltype=ModelType.Add, name='gnei', funcname='gnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='meankT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'grad': XspecModel(modeltype=ModelType.Add, name='grad', funcname='grad', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='D', default=10.0, units='kpc', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='i', default=0.0, units='deg', frozen=True, softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Mass', default=1.0, units='solar', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mdot', default=1.0, units='1e18', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='TclovTef', default=1.7, frozen=True, softmin=1.0, softmax=10.0, hardmin=1.0, hardmax=10.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='refflag', default=1.0, frozen=True, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=1.0)]),
    'grbcomp': XspecModel(modeltype=ModelType.Add, name='grbcomp', funcname='xsgrbcomp', language=LanguageStyle.CStyle8, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kTs', default=1.0, units='keV', softmin=0.0, softmax=20.0, hardmin=0.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='gamma', default=3.0, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kTe', default=100.0, units='keV', softmin=0.2, softmax=2000.0, hardmin=0.2, hardmax=2000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='tau', default=5.0, softmin=0.0, softmax=200.0, hardmin=0.0, hardmax=200.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='beta', default=0.2, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='fbflag', default=0.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='log_A', default=5.0, frozen=True, softmin=-8.0, softmax=8.0, hardmin=-8.0, hardmax=8.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='z', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='a_boost', default=5.0, frozen=True, softmin=0.0, softmax=30.0, hardmin=0.0, hardmax=30.0, delta=0.01)]),
    'grbjet': XspecModel(modeltype=ModelType.Add, name='grbjet', funcname='xsgrbjet', language=LanguageStyle.CStyle8, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='thobs', default=5.0, frozen=True, softmin=0.0, softmax=30.0, hardmin=0.0, hardmax=30.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='thjet', default=10.0, frozen=True, softmin=2.0, softmax=20.0, hardmin=2.0, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='gamma', default=200.0, softmin=1.0, softmax=500.0, hardmin=1.0, hardmax=500.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='r12', default=1.0, frozen=True, softmin=0.1, softmax=100.0, hardmin=0.1, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='p1', default=0.0, softmin=-2.0, softmax=1.0, hardmin=-2.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='p2', default=1.5, softmin=1.1, softmax=10.0, hardmin=1.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='E0', default=1.0, units='keV', softmin=0.1, softmax=1000.0, hardmin=0.1, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='delta', default=0.2, frozen=True, softmin=0.01, softmax=1.5, hardmin=0.01, hardmax=1.5, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='index_pl', default=0.8, frozen=True, softmin=0.0, softmax=1.5, hardmin=0.0, hardmax=1.5, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='ecut', default=20.0, units='keV', frozen=True, softmin=0.1, softmax=1000.0, hardmin=0.1, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='ktbb', default=1.0, units='keV', frozen=True, softmin=0.1, softmax=1000.0, hardmin=0.1, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='model', default=1, frozen=True), XspecParameter(paramtype=ParamType.Default, name='redshift', default=2.0, frozen=True, softmin=0.01, softmax=10.0, hardmin=0.001, hardmax=10.0, delta=1.0)]),
    'grbm': XspecModel(modeltype=ModelType.Add, name='grbm', funcname='xsgrbm', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='alpha', default=-1.0, softmin=-3.0, softmax=2.0, hardmin=-10.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='beta', default=-2.0, softmin=-5.0, softmax=2.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='tem', default=300.0, units='keV', softmin=50.0, softmax=1000.0, hardmin=10.0, hardmax=10000.0, delta=10.0)]),
    'hatm': XspecModel(modeltype=ModelType.Add, name='hatm', funcname='hatm', language=LanguageStyle.CppStyle8, elo=0.0, ehi=20.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='T', default=3.0, units='MK', softmin=0.5, softmax=10.0, hardmin=0.5, hardmax=10.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='NSmass', default=1.4, units='Msun', softmin=0.6, softmax=2.8, hardmin=0.6, hardmax=2.8, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='NSrad', default=10.0, units='km', softmin=5.0, softmax=23.0, hardmin=5.0, hardmax=23.0, delta=0.001)]),
    'jet': XspecModel(modeltype=ModelType.Add, name='jet', funcname='jet', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='mass', default=1000000000.0, units='solar', frozen=True, softmin=1.0, softmax=10000000000.0, hardmin=1.0, hardmax=10000000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Dco', default=3350.6, units='Mpc', frozen=True, softmin=1.0, softmax=100000000.0, hardmin=1.0, hardmax=100000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='log_mdot', default=-1.0, units='logL/LEdd', softmin=-5.0, softmax=2.0, hardmin=-5.0, hardmax=2.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='thetaobs', default=3.0, units='deg', frozen=True, softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='BulkG', default=13.0, frozen=True, softmin=1.0, softmax=100.0, hardmin=1.0, hardmax=100.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='phi', default=0.1, units='rad', frozen=True, softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=100.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='zdiss', default=1275.0, units='Rg', frozen=True, softmin=10.0, softmax=10000.0, hardmin=10.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='B', default=2.6, units='Gau', frozen=True, softmin=0.01, softmax=15.0, hardmin=0.01, hardmax=15.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='logPrel', default=43.3, frozen=True, softmin=40.0, softmax=48.0, hardmin=40.0, hardmax=48.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='gmin_inj', default=1.0, frozen=True, softmin=1.0, softmax=1000.0, hardmin=1.0, hardmax=1000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='gbreak', default=300.0, frozen=True, softmin=10.0, softmax=10000.0, hardmin=10.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='gmax', default=3000.0, frozen=True, softmin=1000.0, softmax=1000000.0, hardmin=1000.0, hardmax=1000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='s1', default=1.0, frozen=True, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='s2', default=2.7, frozen=True, softmin=1.0, softmax=5.0, hardmin=1.0, hardmax=5.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='z', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=1.0)]),
    'kerrbb': XspecModel(modeltype=ModelType.Add, name='kerrbb', funcname='kerrbb', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1000000.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='eta', default=0.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='a', default=0.0, softmin=-1.0, softmax=0.9999, hardmin=-1.0, hardmax=0.9999, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='i', default=30.0, units='deg', frozen=True, softmin=0.0, softmax=85.0, hardmin=0.0, hardmax=85.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mbh', default=1.0, units='Msun', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mdd', default=1.0, units='Mdd0', softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Dbh', default=10.0, units='kpc', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='hd', default=1.7, frozen=True, softmin=1.0, softmax=10.0, hardmin=1.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='rflag', default=1, frozen=True), XspecParameter(paramtype=ParamType.Switch, name='lflag', default=0, frozen=True)]),
    'kerrd': XspecModel(modeltype=ModelType.Add, name='kerrd', funcname='kerrd', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='distance', default=1.0, units='kpc', frozen=True, softmin=0.01, softmax=1000.0, hardmin=0.01, hardmax=1000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='TcoloTeff', default=1.5, frozen=True, softmin=1.0, softmax=2.0, hardmin=1.0, hardmax=2.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='M', default=1.0, units='solar', softmin=0.1, softmax=100.0, hardmin=0.1, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Mdot', default=1.0, units='1e18', softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Incl', default=30.0, units='deg', frozen=True, softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Rin', default=1.235, units='Rg', frozen=True, softmin=1.235, softmax=100.0, hardmin=1.235, hardmax=100.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Rout', default=100000.0, units='Rg', frozen=True, softmin=10000.0, softmax=100000000.0, hardmin=10000.0, hardmax=100000000.0, delta=1.0)]),
    'kerrdisk': XspecModel(modeltype=ModelType.Add, name='kerrdisk', funcname='spin', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='lineE', default=6.4, units='keV', frozen=True, softmin=0.1, softmax=100.0, hardmin=0.1, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Index1', default=3.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Index2', default=3.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='r_br_g', default=6.0, frozen=True, softmin=1.0, softmax=400.0, hardmin=1.0, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='a', default=0.998, softmin=0.01, softmax=0.998, hardmin=0.01, hardmax=0.998, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Incl', default=30.0, units='deg', frozen=True, softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Rin_ms', default=1.0, frozen=True, softmin=1.0, softmax=400.0, hardmin=1.0, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout_ms', default=400.0, frozen=True, softmin=1.0, softmax=400.0, hardmin=1.0, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='z', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001)]),
    'kyconv': XspecModel(modeltype=ModelType.Con, name='kyconv', funcname='kyconv', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='a', default=0.9982, units='GM/c', softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.2), XspecParameter(paramtype=ParamType.Default, name='theta_o', default=30.0, units='deg', softmin=0.0, softmax=89.0, hardmin=0.0, hardmax=89.0, delta=5.0), XspecParameter(paramtype=ParamType.Default, name='rin', default=1.0, units='GM/c^2', frozen=True, softmin=1.0, softmax=1000.0, hardmin=1.0, hardmax=1000.0, delta=0.5), XspecParameter(paramtype=ParamType.Default, name='ms', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='rout', default=400.0, units='GM/c^2', frozen=True, softmin=1.0, softmax=1000.0, hardmin=1.0, hardmax=1000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='alpha', default=3.0, frozen=True, softmin=-20.0, softmax=20.0, hardmin=-20.0, hardmax=20.0, delta=0.5), XspecParameter(paramtype=ParamType.Default, name='beta', default=3.0, frozen=True, softmin=-20.0, softmax=20.0, hardmin=-20.0, hardmax=20.0, delta=0.5), XspecParameter(paramtype=ParamType.Default, name='rb', default=400.0, units='GM/c^2', frozen=True, softmin=1.0, softmax=1000.0, hardmin=1.0, hardmax=1000.0, delta=0.5), XspecParameter(paramtype=ParamType.Default, name='zshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='limb', default=0.0, frozen=True, softmin=0.0, softmax=2.0, hardmin=0.0, hardmax=2.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='ne_loc', default=100.0, frozen=True, softmin=3.0, softmax=5000.0, hardmin=3.0, hardmax=5000.0, delta=100.0), XspecParameter(paramtype=ParamType.Default, name='normal', default=1.0, frozen=True, softmin=-1.0, softmax=100.0, hardmin=-1.0, hardmax=100.0, delta=1.0)]),
    'kyrline': XspecModel(modeltype=ModelType.Add, name='kyrline', funcname='kyrline', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='a', default=0.9982, units='GM/c', softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.2), XspecParameter(paramtype=ParamType.Default, name='theta_o', default=30.0, units='deg', softmin=0.0, softmax=89.0, hardmin=0.0, hardmax=89.0, delta=5.0), XspecParameter(paramtype=ParamType.Default, name='rin', default=1.0, units='GM/c^2', frozen=True, softmin=1.0, softmax=1000.0, hardmin=1.0, hardmax=1000.0, delta=0.5), XspecParameter(paramtype=ParamType.Default, name='ms', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='rout', default=400.0, units='GM/c^2', frozen=True, softmin=1.0, softmax=1000.0, hardmin=1.0, hardmax=1000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Erest', default=6.4, units='keV', frozen=True, softmin=1.0, softmax=99.0, hardmin=1.0, hardmax=99.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='alpha', default=3.0, frozen=True, softmin=-20.0, softmax=20.0, hardmin=-20.0, hardmax=20.0, delta=0.5), XspecParameter(paramtype=ParamType.Default, name='beta', default=3.0, frozen=True, softmin=-20.0, softmax=20.0, hardmin=-20.0, hardmax=20.0, delta=0.5), XspecParameter(paramtype=ParamType.Default, name='rb', default=400.0, units='GM/c^2', frozen=True, softmin=1.0, softmax=1000.0, hardmin=1.0, hardmax=1000.0, delta=0.5), XspecParameter(paramtype=ParamType.Default, name='zshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='limb', default=1.0, frozen=True, softmin=0.0, softmax=2.0, hardmin=0.0, hardmax=2.0, delta=1.0)]),
    'laor': XspecModel(modeltype=ModelType.Add, name='laor', funcname='laor', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='lineE', default=6.4, units='keV', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Index', default=3.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rin_G', default=1.235, frozen=True, softmin=1.235, softmax=400.0, hardmin=1.235, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout_G', default=400.0, frozen=True, softmin=1.235, softmax=400.0, hardmin=1.235, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Incl', default=30.0, units='deg', frozen=True, softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=1.0)]),
    'laor2': XspecModel(modeltype=ModelType.Add, name='laor2', funcname='laor2', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='lineE', default=6.4, units='keV', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Index', default=3.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rin_G', default=1.235, frozen=True, softmin=1.235, softmax=400.0, hardmin=1.235, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout_G', default=400.0, frozen=True, softmin=1.235, softmax=400.0, hardmin=1.235, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Incl', default=30.0, units='deg', frozen=True, softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Rbreak', default=20.0, frozen=True, softmin=1.235, softmax=400.0, hardmin=1.235, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Index1', default=3.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1)]),
    'logpar': XspecModel(modeltype=ModelType.Add, name='logpar', funcname='logpar', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='alpha', default=1.5, softmin=0.0, softmax=4.0, hardmin=0.0, hardmax=4.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='beta', default=0.2, softmin=-4.0, softmax=4.0, hardmin=-4.0, hardmax=4.0, delta=0.01), XspecParameter(paramtype=ParamType.Scale, name='pivotE', default=1.0, units='keV', frozen=True)]),
    'lorentz': XspecModel(modeltype=ModelType.Add, name='lorentz', funcname='lorentzianLine', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='LineE', default=6.5, units='keV', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Width', default=0.1, units='keV', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.05)]),
    'meka': XspecModel(modeltype=ModelType.Add, name='meka', funcname='meka', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.001, softmax=100.0, hardmin=0.001, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'mekal': XspecModel(modeltype=ModelType.Add, name='mekal', funcname='mekal', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=-1.0)]),
    'mkcflow': XspecModel(modeltype=ModelType.Add, name='mkcflow', funcname='xsmkcf', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='lowT', default=0.1, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='highT', default=4.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=-1.0)]),
    'nei': XspecModel(modeltype=ModelType.Add, name='nei', funcname='nei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'nlapec': XspecModel(modeltype=ModelType.Add, name='nlapec', funcname='nlapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.008, softmax=64.0, hardmin=0.008, hardmax=64.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'npshock': XspecModel(modeltype=ModelType.Add, name='npshock', funcname='npshock', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT_a', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_b', default=0.5, units='keV', softmin=0.01, softmax=79.9, hardmin=0.01, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau_l', default=0.0, units='s/cm^3', frozen=True, softmin=0.0, softmax=50000000000000.0, hardmin=0.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Tau_u', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'nsa': XspecModel(modeltype=ModelType.Add, name='nsa', funcname='nsa', language=LanguageStyle.F77Style4, elo=0.05, ehi=12.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='LogT_eff', default=6.0, units='K', softmin=5.0, softmax=7.0, hardmin=5.0, hardmax=7.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='M_ns', default=1.4, units='Msun', softmin=0.5, softmax=2.5, hardmin=0.5, hardmax=2.5, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='R_ns', default=10.0, units='km', softmin=5.0, softmax=20.0, hardmin=5.0, hardmax=20.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='MagField', default=0.0, units='G', frozen=True, softmin=0.0, softmax=50000000000000.0, hardmin=0.0, hardmax=50000000000000.0, delta=1000000000.0)]),
    'nsagrav': XspecModel(modeltype=ModelType.Add, name='nsagrav', funcname='nsagrav', language=LanguageStyle.F77Style4, elo=0.05, ehi=10.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='LogT_eff', default=6.0, units='K', softmin=5.5, softmax=6.5, hardmin=5.5, hardmax=6.5, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='NSmass', default=1.4, units='Msun', softmin=0.3, softmax=2.5, hardmin=0.3, hardmax=2.5, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='NSrad', default=10.0, units='km', softmin=6.0, softmax=20.0, hardmin=6.0, hardmax=20.0, delta=0.001)]),
    'nsatmos': XspecModel(modeltype=ModelType.Add, name='nsatmos', funcname='nsatmos', language=LanguageStyle.F77Style4, elo=0.05, ehi=10.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='LogT_eff', default=6.0, units='K', softmin=5.0, softmax=6.5, hardmin=5.0, hardmax=6.5, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='M_ns', default=1.4, units='Msun', softmin=0.5, softmax=3.0, hardmin=0.5, hardmax=3.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='R_ns', default=10.0, units='km', softmin=5.0, softmax=30.0, hardmin=5.0, hardmax=30.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='dist', default=10.0, units='kpc', softmin=0.1, softmax=100.0, hardmin=0.1, hardmax=100.0, delta=0.1)]),
    'nsmax': XspecModel(modeltype=ModelType.Add, name='nsmax', funcname='nsmax', language=LanguageStyle.CppStyle8, elo=0.05, ehi=10.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='logTeff', default=6.0, units='K', softmin=5.5, softmax=6.8, hardmin=5.5, hardmax=6.8, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.1, softmin=1e-05, softmax=1.5, hardmin=1e-05, hardmax=2.0, delta=0.1), XspecParameter(paramtype=ParamType.Switch, name='specfile', default=1200, frozen=True)]),
    'nsmaxg': XspecModel(modeltype=ModelType.Add, name='nsmaxg', funcname='nsmaxg', language=LanguageStyle.CppStyle8, elo=0.05, ehi=10.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='logTeff', default=6.0, units='K', softmin=5.5, softmax=6.9, hardmin=5.5, hardmax=6.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='M_ns', default=1.4, units='Msun', softmin=0.5, softmax=3.0, hardmin=0.5, hardmax=3.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='R_ns', default=10.0, units='km', softmin=5.0, softmax=30.0, hardmin=5.0, hardmax=30.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='dist', default=1.0, units='kpc', softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Switch, name='specfile', default=1200, frozen=True)]),
    'nsx': XspecModel(modeltype=ModelType.Add, name='nsx', funcname='nsx', language=LanguageStyle.CppStyle8, elo=0.05, ehi=10.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='logTeff', default=6.0, units='K', softmin=5.5, softmax=6.7, hardmin=5.5, hardmax=6.7, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='M_ns', default=1.4, units='Msun', softmin=0.5, softmax=3.0, hardmin=0.5, hardmax=3.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='R_ns', default=10.0, units='km', softmin=5.0, softmax=30.0, hardmin=5.0, hardmax=30.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='dist', default=1.0, units='kpc', softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Switch, name='specfile', default=6, frozen=True)]),
    'nteea': XspecModel(modeltype=ModelType.Add, name='nteea', funcname='xsnteea', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='l_nth', default=100.0, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='l_bb', default=100.0, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='f_refl', default=0.0, softmin=0.0, softmax=4.0, hardmin=0.0, hardmax=4.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='kT_bb', default=10.0, frozen=True, softmin=1.0, softmax=100.0, hardmin=1.0, hardmax=100.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='g_max', default=1000.0, frozen=True, softmin=5.0, softmax=10000.0, hardmin=5.0, hardmax=10000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='l_th', default=0.0, frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='tau_p', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.2), XspecParameter(paramtype=ParamType.Default, name='G_inj', default=0.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='g_min', default=1.3, frozen=True, softmin=1.0, softmax=1000.0, hardmin=1.0, hardmax=1000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='g_0', default=1.3, frozen=True, softmin=1.0, softmax=5.0, hardmin=1.0, hardmax=5.0, delta=0.5), XspecParameter(paramtype=ParamType.Default, name='radius', default=10000000000000.0, frozen=True, softmin=100000.0, softmax=1e+16, hardmin=100000.0, hardmax=1e+16, delta=100000.0), XspecParameter(paramtype=ParamType.Default, name='pair_esc', default=0.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.45, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'nthComp': XspecModel(modeltype=ModelType.Add, name='nthComp', funcname='nthcomp', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Gamma', default=1.7, softmin=1.001, softmax=5.0, hardmin=1.001, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_e', default=100.0, units='keV', softmin=5.0, softmax=1000.0, hardmin=1.0, hardmax=1000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='kT_bb', default=0.1, units='keV', frozen=True, softmin=0.001, softmax=10.0, hardmin=0.001, hardmax=10.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='inp_type', default=0.0, units='0/1', frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'optxagn': XspecModel(modeltype=ModelType.Add, name='optxagn', funcname='optxagn', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='mass', default=10000000.0, units='solar', frozen=True, softmin=1.0, softmax=1000000000.0, hardmin=1.0, hardmax=1000000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='dist', default=100.0, units='Mpc', frozen=True, softmin=0.01, softmax=1000000000.0, hardmin=0.01, hardmax=1000000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='logLoLEdd', default=-1.0, softmin=-10.0, softmax=2.0, hardmin=-10.0, hardmax=2.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='astar', default=0.0, frozen=True, softmin=0.0, softmax=0.998, hardmin=0.0, hardmax=0.998, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='rcor', default=10.0, units='rg', softmin=1.0, softmax=100.0, hardmin=1.0, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='logrout', default=5.0, frozen=True, softmin=3.0, softmax=7.0, hardmin=3.0, hardmax=7.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_e', default=0.2, units='keV', softmin=0.01, softmax=10.0, hardmin=0.01, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='tau', default=10.0, softmin=0.1, softmax=100.0, hardmin=0.1, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Gamma', default=2.1, softmin=0.5, softmax=5.0, hardmin=0.5, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='fpl', default=0.0001, softmin=0.0, softmax=0.1, hardmin=0.0, hardmax=0.1, delta=1e-06), XspecParameter(paramtype=ParamType.Default, name='fcol', default=2.4, frozen=True, softmin=1.0, softmax=5.0, hardmin=1.0, hardmax=5.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='tscat', default=100000.0, frozen=True, softmin=10000.0, softmax=100000.0, hardmin=10000.0, hardmax=100000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01)]),
    'optxagnf': XspecModel(modeltype=ModelType.Add, name='optxagnf', funcname='optxagnf', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='mass', default=10000000.0, units='solar', frozen=True, softmin=1.0, softmax=1000000000.0, hardmin=1.0, hardmax=1000000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='dist', default=100.0, units='Mpc', frozen=True, softmin=0.01, softmax=1000000000.0, hardmin=0.01, hardmax=1000000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='logLoLEdd', default=-1.0, softmin=-10.0, softmax=2.0, hardmin=-10.0, hardmax=2.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='astar', default=0.0, frozen=True, softmin=0.0, softmax=0.998, hardmin=0.0, hardmax=0.998, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='rcor', default=10.0, units='rg', softmin=1.0, softmax=100.0, hardmin=1.0, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='logrout', default=5.0, frozen=True, softmin=3.0, softmax=7.0, hardmin=3.0, hardmax=7.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_e', default=0.2, units='keV', softmin=0.01, softmax=10.0, hardmin=0.01, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='tau', default=10.0, softmin=0.1, softmax=100.0, hardmin=0.1, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Gamma', default=2.1, softmin=1.05, softmax=5.0, hardmin=1.05, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='fpl', default=0.0001, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=1e-06), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01)]),
    'pegpwrlw': XspecModel(modeltype=ModelType.Add, name='pegpwrlw', funcname='xspegp', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=1.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='eMin', default=2.0, units='keV', frozen=True, softmin=-100.0, softmax=10000000000.0, hardmin=-100.0, hardmax=10000000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='eMax', default=10.0, units='keV', frozen=True, softmin=-100.0, softmax=10000000000.0, hardmin=-100.0, hardmax=10000000000.0, delta=0.01)]),
    'pexmon': XspecModel(modeltype=ModelType.Add, name='pexmon', funcname='pexmon', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=2.0, softmin=1.1, softmax=2.5, hardmin=1.1, hardmax=2.5, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='foldE', default=1000.0, units='keV', frozen=True, softmin=1.0, softmax=1000000.0, hardmin=1.0, hardmax=1000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=-1.0, frozen=True, softmin=-1000000.0, softmax=1000000.0, hardmin=-1000000.0, hardmax=1000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=4.0, hardmin=0.0, hardmax=4.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Incl', default=60.0, units='deg', softmin=0.0, softmax=85.0, hardmin=0.0, hardmax=85.0, delta=1.0)]),
    'pexrav': XspecModel(modeltype=ModelType.Add, name='pexrav', funcname='xspexrav', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=2.0, softmin=-9.0, softmax=9.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='foldE', default=100.0, units='keV', softmin=1.0, softmax=1000000.0, hardmin=1.0, hardmax=1000000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=0.0, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.45, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.01)]),
    'pexriv': XspecModel(modeltype=ModelType.Add, name='pexriv', funcname='xspexriv', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=2.0, softmin=-9.0, softmax=9.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='foldE', default=100.0, units='keV', softmin=1.0, softmax=1000000.0, hardmin=1.0, hardmax=1000000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=0.0, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.45, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='T_disk', default=30000.0, units='K', frozen=True, softmin=10000.0, softmax=1000000.0, hardmin=10000.0, hardmax=1000000.0, delta=1000.0), XspecParameter(paramtype=ParamType.Default, name='xi', default=1.0, units='erg cm/s', softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=5000.0, delta=0.1)]),
    'plcabs': XspecModel(modeltype=ModelType.Add, name='plcabs', funcname='xsp1tr', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Scale, name='nmax', default=1.0, frozen=True), XspecParameter(paramtype=ParamType.Default, name='FeAbun', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='FeKedge', default=7.11, units='KeV', frozen=True, softmin=7.0, softmax=10.0, hardmin=7.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=2.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='HighECut', default=95.0, units='keV', frozen=True, softmin=1.0, softmax=100.0, hardmin=0.01, hardmax=200.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='foldE', default=100.0, frozen=True, softmin=1.0, softmax=1000000.0, hardmin=1.0, hardmax=1000000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='acrit', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Scale, name='FAST', default=0.0, frozen=True), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'powerlaw': XspecModel(modeltype=ModelType.Add, name='powerlaw', funcname='powerLaw', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=1.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01)]),
    'posm': XspecModel(modeltype=ModelType.Add, name='posm', funcname='xsposm', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[]),
    'pshock': XspecModel(modeltype=ModelType.Add, name='pshock', funcname='pshock', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau_l', default=0.0, units='s/cm^3', frozen=True, softmin=0.0, softmax=50000000000000.0, hardmin=0.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Tau_u', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'qsosed': XspecModel(modeltype=ModelType.Add, name='qsosed', funcname='qsosed', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='mass', default=10000000.0, units='solar', frozen=True, softmin=100000.0, softmax=10000000000.0, hardmin=100000.0, hardmax=10000000000.0, delta=1000.0), XspecParameter(paramtype=ParamType.Default, name='dist', default=100.0, units='Mpc', frozen=True, softmin=0.01, softmax=1000000000.0, hardmin=0.01, hardmax=1000000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='logmdot', default=-1.0, units='Ledd', softmin=-1.65, softmax=0.39, hardmin=-1.65, hardmax=0.39, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='astar', default=0.0, frozen=True, softmin=-1.0, softmax=0.998, hardmin=-1.0, hardmax=0.998, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='cosi', default=0.5, frozen=True, softmin=0.05, softmax=1.0, hardmin=0.05, hardmax=1.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=1.0)]),
    'raymond': XspecModel(modeltype=ModelType.Add, name='raymond', funcname='raysmith', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.008, softmax=64.0, hardmin=0.008, hardmax=64.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'redge': XspecModel(modeltype=ModelType.Add, name='redge', funcname='xredge', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='edge', default=1.4, units='keV', softmin=0.001, softmax=100.0, hardmin=0.001, hardmax=100.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.001, softmax=100.0, hardmin=0.001, hardmax=100.0, delta=0.001)]),
    'refsch': XspecModel(modeltype=ModelType.Add, name='refsch', funcname='xsrefsch', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=2.0, softmin=-9.0, softmax=9.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='foldE', default=100.0, units='keV', softmin=1.0, softmax=1000000.0, hardmin=1.0, hardmax=1000000.0, delta=10.0), XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=0.0, softmin=0.0, softmax=2.0, hardmin=0.0, hardmax=2.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abund', default=1.0, frozen=True, softmin=0.5, softmax=10.0, hardmin=0.5, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.1, softmax=10.0, hardmin=0.1, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Incl', default=30.0, units='deg', frozen=True, softmin=19.0, softmax=87.0, hardmin=19.0, hardmax=87.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='T_disk', default=30000.0, units='K', frozen=True, softmin=10000.0, softmax=1000000.0, hardmin=10000.0, hardmax=1000000.0, delta=1000.0), XspecParameter(paramtype=ParamType.Default, name='xi', default=1.0, units='ergcm/s', softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=5000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Betor10', default=-2.0, frozen=True, softmin=-10.0, softmax=20.0, hardmin=-10.0, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rin', default=10.0, units='R_g', frozen=True, softmin=6.0, softmax=1000.0, hardmin=6.0, hardmax=10000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout', default=1000.0, units='R_g', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=10000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='accuracy', default=30.0, frozen=True, softmin=30.0, softmax=100000.0, hardmin=30.0, hardmax=100000.0, delta=1.0)]),
    'rnei': XspecModel(modeltype=ModelType.Add, name='rnei', funcname='rnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=0.5, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_init', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'sedov': XspecModel(modeltype=ModelType.Add, name='sedov', funcname='sedov', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT_a', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_b', default=0.5, units='keV', softmin=0.01, softmax=79.9, hardmin=0.01, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'sirf': XspecModel(modeltype=ModelType.Add, name='sirf', funcname='sirf', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='tin', default=1.0, units='keV', softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='rin', default=0.01, units='rsph', softmin=1e-05, softmax=1.0, hardmin=1e-06, hardmax=10.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='rout', default=100.0, units='rsph', softmin=0.1, softmax=100000000.0, hardmin=0.1, hardmax=100000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='theta', default=22.9, units='deg', softmin=1.0, softmax=89.0, hardmin=0.0, hardmax=90.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='incl', default=0.0, units='deg', frozen=True, softmin=-90.0, softmax=90.0, hardmin=-90.0, hardmax=90.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='valpha', default=-0.5, frozen=True, softmin=-1.0, softmax=2.0, hardmin=-1.5, hardmax=5.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='gamma', default=1.333, frozen=True, softmin=0.5, softmax=10.0, hardmin=0.5, hardmax=10.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='mdot', default=1000.0, frozen=True, softmin=0.5, softmax=10000000.0, hardmin=0.5, hardmax=10000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='irrad', default=2.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=1.0)]),
    'slimbh': XspecModel(modeltype=ModelType.Add, name='slimbh', funcname='slimbbmodel', language=LanguageStyle.CStyle8, elo=1000.0, ehi=100000.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='M', default=10.0, units='Msun', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='a', default=0.0, units='GM/c', softmin=0.0, softmax=0.999, hardmin=0.0, hardmax=0.999, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='lumin', default=0.5, units='L_Edd', softmin=0.05, softmax=1.0, hardmin=0.05, hardmax=1.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='alpha', default=0.1, frozen=True, softmin=0.005, softmax=0.1, hardmin=0.005, hardmax=0.1, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='inc', default=60.0, units='deg', frozen=True, softmin=0.0, softmax=85.0, hardmin=0.0, hardmax=85.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='D', default=10.0, units='kpc', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='f_hard', default=-1.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=1.0), XspecParameter(paramtype=ParamType.Switch, name='lflag', default=1, frozen=True), XspecParameter(paramtype=ParamType.Switch, name='vflag', default=1, frozen=True)]),
    'smaug': XspecModel(modeltype=ModelType.Add, name='smaug', funcname='xsmaug', language=LanguageStyle.CStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT_cc', default=1.0, units='keV', softmin=0.1, softmax=10.0, hardmin=0.08, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_dt', default=1.0, units='keV', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_ix', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='kT_ir', default=0.1, units='Mpc', frozen=True, softmin=0.0001, softmax=1.0, hardmin=0.0001, hardmax=1.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='kT_cx', default=0.5, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='kT_cr', default=0.1, units='Mpc', softmin=0.0001, softmax=10.0, hardmin=0.0001, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_tx', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='kT_tr', default=0.5, units='Mpc', frozen=True, softmin=0.0001, softmax=1.0, hardmin=0.0001, hardmax=3.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH_cc', default=1.0, units='cm**-3', frozen=True, softmin=1e-06, softmax=3.0, hardmin=1e-06, hardmax=3.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH_ff', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH_cx', default=0.5, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='nH_cr', default=0.1, units='Mpc', softmin=0.0001, softmax=1.0, hardmin=0.0001, hardmax=2.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH_gx', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='nH_gr', default=0.002, units='Mpc', frozen=True, softmin=0.0001, softmax=10.0, hardmin=0.0001, hardmax=20.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Ab_cc', default=1.0, units='solar', frozen=True, softmin=0.0, softmax=3.0, hardmin=0.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ab_xx', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Ab_rr', default=0.1, units='Mpc', frozen=True, softmin=0.0001, softmax=1.0, hardmin=0.0001, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.01, frozen=True, softmin=0.0001, softmax=10.0, hardmin=0.0001, hardmax=10.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='meshpts', default=10.0, frozen=True, softmin=1.0, softmax=10000.0, hardmin=1.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='rcutoff', default=2.0, units='Mpc', frozen=True, softmin=1.0, softmax=3.0, hardmin=1.0, hardmax=3.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='mode', default=1.0, frozen=True, softmin=0.0, softmax=2.0, hardmin=0.0, hardmax=2.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='itype', default=2.0, frozen=True, softmin=1.0, softmax=4.0, hardmin=1.0, hardmax=4.0, delta=1.0)], can_cache=False),
    'snapec': XspecModel(modeltype=ModelType.Add, name='snapec', funcname='snapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.008, softmax=64.0, hardmin=0.008, hardmax=64.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N_SNe', default=1.0, units='10^9', softmin=0.0, softmax=1e+20, hardmin=0.0, hardmax=1e+20, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='R', default=1.0, softmin=0.0, softmax=1e+20, hardmin=0.0, hardmax=1e+20, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='SNIModelIndex', default=1.0, frozen=True, softmin=0.0, softmax=125.0, hardmin=0.0, hardmax=125.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='SNIIModelIndex', default=1.0, frozen=True, softmin=0.0, softmax=125.0, hardmin=0.0, hardmax=125.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01)]),
    'srcut': XspecModel(modeltype=ModelType.Add, name='srcut', funcname='srcut', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='alpha', default=0.5, softmin=0.3, softmax=0.8, hardmin=1e-05, hardmax=1.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='break_', default=2.42e+17, units='Hz', softmin=1000000000000000.0, softmax=1e+19, hardmin=10000000000.0, hardmax=1e+25, delta=10000000000.0)]),
    'sresc': XspecModel(modeltype=ModelType.Add, name='sresc', funcname='sresc', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='alpha', default=0.5, softmin=0.3, softmax=0.8, hardmin=1e-05, hardmax=1.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='rolloff', default=2.42e+17, units='Hz', softmin=1000000000000000.0, softmax=1e+19, hardmin=10000000000.0, hardmax=1e+25, delta=10000000000.0)]),
    'ssa': XspecModel(modeltype=ModelType.Add, name='ssa', funcname='ssa', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='te', default=0.1, softmin=0.01, softmax=0.5, hardmin=0.01, hardmax=0.5, delta=1e-05), XspecParameter(paramtype=ParamType.Default, name='y', default=0.7, softmin=0.0001, softmax=1000.0, hardmin=0.0001, hardmax=1000.0, delta=1e-05)]),
    'step': XspecModel(modeltype=ModelType.Add, name='step', funcname='xsstep', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Energy', default=6.5, units='keV', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Sigma', default=0.1, units='keV', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.05)]),
    'tapec': XspecModel(modeltype=ModelType.Add, name='tapec', funcname='tapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.008, softmax=64.0, hardmin=0.008, hardmax=64.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kTi', default=1.0, units='keV', softmin=0.008, softmax=64.0, hardmin=0.008, hardmax=64.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Abundanc', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vapec': XspecModel(modeltype=ModelType.Add, name='vapec', funcname='vapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vbremss': XspecModel(modeltype=ModelType.Add, name='vbremss', funcname='xsbrmv', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=3.0, units='keV', softmin=0.01, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='HeovrH', default=1.0, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01)]),
    'vcph': XspecModel(modeltype=ModelType.Add, name='vcph', funcname='vcph', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='peakT', default=2.2, units='keV', softmin=0.1, softmax=100.0, hardmin=0.1, hardmax=100.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=0.0, softmax=50.0, hardmin=0.0, hardmax=50.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True)]),
    'vequil': XspecModel(modeltype=ModelType.Add, name='vequil', funcname='vequil', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vgadem': XspecModel(modeltype=ModelType.Add, name='vgadem', funcname='vgaussDem', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Tmean', default=4.0, units='keV', frozen=True, softmin=0.01, softmax=10.0, hardmin=0.01, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tsigma', default=0.1, units='keV', softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=2, frozen=True)]),
    'vgnei': XspecModel(modeltype=ModelType.Add, name='vgnei', funcname='vgnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='meankT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vmeka': XspecModel(modeltype=ModelType.Add, name='vmeka', funcname='vmeka', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.001, softmax=100.0, hardmin=0.001, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vmekal': XspecModel(modeltype=ModelType.Add, name='vmekal', funcname='vmekal', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True)]),
    'vmcflow': XspecModel(modeltype=ModelType.Add, name='vmcflow', funcname='xsvmcf', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='lowT', default=0.1, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='highT', default=4.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=1, frozen=True)]),
    'vnei': XspecModel(modeltype=ModelType.Add, name='vnei', funcname='vnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vnpshock': XspecModel(modeltype=ModelType.Add, name='vnpshock', funcname='vnpshock', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT_a', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_b', default=0.5, units='keV', softmin=0.01, softmax=79.9, hardmin=0.01, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau_l', default=0.0, units='s/cm^3', frozen=True, softmin=0.0, softmax=50000000000000.0, hardmin=0.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Tau_u', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'voigt': XspecModel(modeltype=ModelType.Add, name='voigt', funcname='voigtLine', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='LineE', default=6.5, units='keV', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sigma', default=0.01, units='keV', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.005), XspecParameter(paramtype=ParamType.Default, name='Gamma', default=0.01, units='keV', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.005)]),
    'vpshock': XspecModel(modeltype=ModelType.Add, name='vpshock', funcname='vpshock', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau_l', default=0.0, units='s/cm^3', frozen=True, softmin=0.0, softmax=50000000000000.0, hardmin=0.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Tau_u', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vraymond': XspecModel(modeltype=ModelType.Add, name='vraymond', funcname='vraysmith', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=6.5, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vrnei': XspecModel(modeltype=ModelType.Add, name='vrnei', funcname='vrnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=0.5, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_init', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vsedov': XspecModel(modeltype=ModelType.Add, name='vsedov', funcname='vsedov', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT_a', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_b', default=0.5, units='keV', softmin=0.01, softmax=79.9, hardmin=0.01, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vtapec': XspecModel(modeltype=ModelType.Add, name='vtapec', funcname='vtapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kTi', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vvapec': XspecModel(modeltype=ModelType.Add, name='vvapec', funcname='vvapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vvgnei': XspecModel(modeltype=ModelType.Add, name='vvgnei', funcname='vvgnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='meankT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vvnei': XspecModel(modeltype=ModelType.Add, name='vvnei', funcname='vvnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vvnpshock': XspecModel(modeltype=ModelType.Add, name='vvnpshock', funcname='vvnpshock', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT_a', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_b', default=0.5, units='keV', softmin=0.01, softmax=79.9, hardmin=0.01, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau_l', default=0.0, units='s/cm^3', frozen=True, softmin=0.0, softmax=50000000000000.0, hardmin=0.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Tau_u', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vvpshock': XspecModel(modeltype=ModelType.Add, name='vvpshock', funcname='vvpshock', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau_l', default=0.0, units='s/cm^3', frozen=True, softmin=0.0, softmax=50000000000000.0, hardmin=0.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Tau_u', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vvrnei': XspecModel(modeltype=ModelType.Add, name='vvrnei', funcname='vvrnei', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=0.5, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_init', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vvsedov': XspecModel(modeltype=ModelType.Add, name='vvsedov', funcname='vvsedov', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT_a', default=1.0, units='keV', softmin=0.0808, softmax=79.9, hardmin=0.0808, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_b', default=0.5, units='keV', softmin=0.01, softmax=79.9, hardmin=0.01, hardmax=79.9, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Tau', default=100000000000.0, units='s/cm^3', softmin=100000000.0, softmax=50000000000000.0, hardmin=100000000.0, hardmax=50000000000000.0, delta=100000000.0), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vvtapec': XspecModel(modeltype=ModelType.Add, name='vvtapec', funcname='vvtapec', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kTi', default=6.5, units='keV', softmin=0.0808, softmax=68.447, hardmin=0.0808, hardmax=68.447, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'vvwdem': XspecModel(modeltype=ModelType.Add, name='vvwdem', funcname='vvwDem', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Tmax', default=1.0, units='keV', softmin=0.01, softmax=10.0, hardmin=0.01, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='beta', default=0.1, softmin=0.01, softmax=1.0, hardmin=0.01, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='inv_slope', default=0.25, softmin=-1.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Li', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Be', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='F', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='P', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='K', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sc', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ti', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='V', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cu', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Zn', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=2, frozen=True)]),
    'vwdem': XspecModel(modeltype=ModelType.Add, name='vwdem', funcname='vwDem', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Tmax', default=1.0, units='keV', softmin=0.01, softmax=10.0, hardmin=0.01, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='beta', default=0.1, softmin=0.01, softmax=1.0, hardmin=0.01, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='inv_slope', default=0.25, softmin=-1.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=2, frozen=True)]),
    'wdem': XspecModel(modeltype=ModelType.Add, name='wdem', funcname='wDem', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Tmax', default=1.0, units='keV', softmin=0.01, softmax=10.0, hardmin=0.01, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='beta', default=0.1, softmin=0.01, softmax=1.0, hardmin=0.01, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='inv_slope', default=0.25, softmin=-1.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='cm^-3', frozen=True, softmin=1e-05, softmax=1e+19, hardmin=1e-06, hardmax=1e+20, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abundanc', default=1.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='switch', default=2, frozen=True)]),
    'zagauss': XspecModel(modeltype=ModelType.Add, name='zagauss', funcname='zagauss', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='LineE', default=10.0, units='A', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Sigma', default=1.0, units='A', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zbbody': XspecModel(modeltype=ModelType.Add, name='zbbody', funcname='xszbod', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=3.0, units='keV', softmin=0.01, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zbknpower': XspecModel(modeltype=ModelType.Add, name='zbknpower', funcname='zBrokenPowerLaw', language=LanguageStyle.CppStyle8, elo=-1e+20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndx1', default=1.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='BreakE', default=5.0, units='keV', softmin=0.01, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='PhoIndx2', default=2.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zbremss': XspecModel(modeltype=ModelType.Add, name='zbremss', funcname='xszbrm', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='kT', default=7.0, units='keV', softmin=0.0001, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zcutoffpl': XspecModel(modeltype=ModelType.Add, name='zcutoffpl', funcname='zcutoffPowerLaw', language=LanguageStyle.CppStyle8, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=1.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='HighECut', default=15.0, units='keV', softmin=1.0, softmax=500.0, hardmin=0.01, hardmax=500.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zgauss': XspecModel(modeltype=ModelType.Add, name='zgauss', funcname='xszgau', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='LineE', default=6.5, units='keV', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Sigma', default=0.1, units='keV', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zkerrbb': XspecModel(modeltype=ModelType.Add, name='zkerrbb', funcname='zkerrbb', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1000000.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='eta', default=0.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='a', default=0.5, softmin=-0.99, softmax=0.9999, hardmin=-0.99, hardmax=0.9999, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='i', default=30.0, units='deg', frozen=True, softmin=0.0, softmax=85.0, hardmin=0.0, hardmax=85.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mbh', default=10000000.0, units='M_sun', softmin=3.0, softmax=10000000000.0, hardmin=3.0, hardmax=10000000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mdd', default=1.0, units='M0yr', softmin=0.0001, softmax=10000.0, hardmin=1e-05, hardmax=100000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='z', default=0.01, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='fcol', default=2.0, frozen=True, softmin=-100.0, softmax=100.0, hardmin=-100.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Switch, name='rflag', default=1, frozen=True), XspecParameter(paramtype=ParamType.Switch, name='lflag', default=1, frozen=True)]),
    'zlogpar': XspecModel(modeltype=ModelType.Add, name='zlogpar', funcname='zLogpar', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='alpha', default=1.5, softmin=0.0, softmax=4.0, hardmin=0.0, hardmax=4.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='beta', default=0.2, softmin=-4.0, softmax=4.0, hardmin=-4.0, hardmax=4.0, delta=0.01), XspecParameter(paramtype=ParamType.Scale, name='pivotE', default=1.0, units='keV', frozen=True), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zpowerlw': XspecModel(modeltype=ModelType.Add, name='zpowerlw', funcname='zpowerLaw', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=1.0, softmin=-2.0, softmax=9.0, hardmin=-3.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'absori': XspecModel(modeltype=ModelType.Mul, name='absori', funcname='xsabsori', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='PhoIndex', default=2.0, frozen=True, softmin=0.0, softmax=4.0, hardmin=0.0, hardmax=4.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Temp_abs', default=30000.0, units='K', frozen=True, softmin=10000.0, softmax=1000000.0, hardmin=10000.0, hardmax=1000000.0, delta=1000.0), XspecParameter(paramtype=ParamType.Default, name='xi', default=1.0, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=5000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.02)]),
    'acisabs': XspecModel(modeltype=ModelType.Mul, name='acisabs', funcname='acisabs', language=LanguageStyle.CppStyle8, elo=0.03, ehi=15.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='Tdays', default=850.0, units='days', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='norm', default=0.00722, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=1e-05), XspecParameter(paramtype=ParamType.Default, name='tauinf', default=0.582, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='tefold', default=620.0, units='days', frozen=True, softmin=1.0, softmax=10000.0, hardmin=1.0, hardmax=10000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='nC', default=10.0, frozen=True, softmin=0.0, softmax=50.0, hardmin=0.0, hardmax=50.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='nH', default=20.0, frozen=True, softmin=1.0, softmax=50.0, hardmin=1.0, hardmax=50.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='nO', default=2.0, frozen=True, softmin=0.0, softmax=50.0, hardmin=0.0, hardmax=50.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='nN', default=1.0, frozen=True, softmin=0.0, softmax=50.0, hardmin=0.0, hardmax=50.0, delta=1.0)]),
    'constant': XspecModel(modeltype=ModelType.Mul, name='constant', funcname='xscnst', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='factor', default=1.0, softmin=0.0, softmax=10000000000.0, hardmin=0.0, hardmax=10000000000.0, delta=0.01)]),
    'cabs': XspecModel(modeltype=ModelType.Mul, name='cabs', funcname='xscabs', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001)]),
    'cyclabs': XspecModel(modeltype=ModelType.Mul, name='cyclabs', funcname='xscycl', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Depth0', default=2.0, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='E0', default=30.0, units='keV', softmin=1.0, softmax=100.0, hardmin=1.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Width0', default=10.0, units='keV', frozen=True, softmin=1.0, softmax=100.0, hardmin=1.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Depth2', default=0.0, frozen=True, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Width2', default=20.0, units='keV', frozen=True, softmin=1.0, softmax=100.0, hardmin=1.0, hardmax=100.0, delta=0.01)]),
    'dust': XspecModel(modeltype=ModelType.Mul, name='dust', funcname='xsdust', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Frac', default=0.066, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Halosz', default=2.0, frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=100000.0, delta=0.01)]),
    'edge': XspecModel(modeltype=ModelType.Mul, name='edge', funcname='xsedge', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='edgeE', default=7.0, units='keV', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='MaxTau', default=1.0, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=10.0, delta=0.01)]),
    'expabs': XspecModel(modeltype=ModelType.Mul, name='expabs', funcname='xsabsc', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='LowECut', default=2.0, units='keV', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=200.0, delta=0.01)]),
    'expfac': XspecModel(modeltype=ModelType.Mul, name='expfac', funcname='xsexp', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Ampl', default=1.0, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Factor', default=1.0, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='StartE', default=0.5, units='keV', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01)]),
    'gabs': XspecModel(modeltype=ModelType.Mul, name='gabs', funcname='gaussianAbsorptionLine', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='LineE', default=1.0, units='keV', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Sigma', default=0.01, units='keV', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Strength', default=1.0, units='keV', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.05)]),
    'heilin': XspecModel(modeltype=ModelType.Mul, name='heilin', funcname='xsphei', language=LanguageStyle.F77Style4, elo=0.01, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nHeI', default=1e-05, units='10^22', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=1e-07), XspecParameter(paramtype=ParamType.Default, name='b', default=10.0, units='km/s', softmin=1.0, softmax=100000.0, hardmin=1.0, hardmax=1000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='z', default=0.0, softmin=-0.001, softmax=100000.0, hardmin=-0.001, hardmax=100000.0, delta=1e-06)]),
    'highecut': XspecModel(modeltype=ModelType.Mul, name='highecut', funcname='xshecu', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='cutoffE', default=10.0, units='keV', softmin=0.01, softmax=1000000.0, hardmin=0.0001, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='foldE', default=15.0, units='keV', softmin=0.01, softmax=1000000.0, hardmin=0.0001, hardmax=1000000.0, delta=0.01)]),
    'hrefl': XspecModel(modeltype=ModelType.Mul, name='hrefl', funcname='xshrfl', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='thetamin', default=0.0, frozen=True, softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='thetamax', default=90.0, frozen=True, softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='thetaobs', default=60.0, softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Feabun', default=1.0, frozen=True, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=200.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='FeKedge', default=7.11, units='keV', frozen=True, softmin=7.0, softmax=10.0, hardmin=7.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Escfrac', default=1.0, softmin=0.0, softmax=500.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='covfac', default=1.0, softmin=0.0, softmax=500.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'ismabs': XspecModel(modeltype=ModelType.Mul, name='ismabs', funcname='ismabs', language=LanguageStyle.F77Style8, elo=0.01, ehi=1000000.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='H', default=0.1, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='He_II', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C_I', default=33.1, units='10^16', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='C_II', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C_III', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N_I', default=8.32, units='10^16', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='N_II', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N_III', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O_I', default=67.6, units='10^16', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='O_II', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O_III', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne_I', default=12.0, units='10^16', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Ne_II', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne_III', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg_I', default=3.8, units='10^16', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Mg_II', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg_III', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si_I', default=3.35, units='10^16', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Si_II', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si_III', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S_I', default=2.14, units='10^16', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='S_II', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S_III', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar_I', default=0.25, units='10^16', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Ar_II', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar_III', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca_I', default=0.22, units='10^16', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Ca_II', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca_III', default=0.0, units='10^16', frozen=True, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=3.16, units='10^16', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.01)]),
    'ismdust': XspecModel(modeltype=ModelType.Mul, name='ismdust', funcname='ismdust', language=LanguageStyle.F77Style8, elo=0.01, ehi=1000000.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='msil', default=1.0, units='10^-4', softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=100000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='mgra', default=1.0, units='10^-4', softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=100000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.01)]),
    'logconst': XspecModel(modeltype=ModelType.Mul, name='logconst', funcname='logconst', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='logfact', default=0.0, softmin=-20.0, softmax=20.0, hardmin=-20.0, hardmax=20.0, delta=0.01)]),
    'log10con': XspecModel(modeltype=ModelType.Mul, name='log10con', funcname='log10con', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='log10fac', default=0.0, softmin=-20.0, softmax=20.0, hardmin=-20.0, hardmax=20.0, delta=0.01)]),
    'lyman': XspecModel(modeltype=ModelType.Mul, name='lyman', funcname='xslyman', language=LanguageStyle.F77Style4, elo=0.01, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='n', default=1e-05, units='10^22', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=1e-07), XspecParameter(paramtype=ParamType.Default, name='b', default=10.0, units='km/s', softmin=1.0, softmax=100000.0, hardmin=1.0, hardmax=1000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='z', default=0.0, softmin=-0.001, softmax=100000.0, hardmin=-0.001, hardmax=100000.0, delta=1e-06), XspecParameter(paramtype=ParamType.Default, name='ZA', default=1.0, softmin=1.0, softmax=2.0, hardmin=1.0, hardmax=2.0, delta=0.1)]),
    'notch': XspecModel(modeltype=ModelType.Mul, name='notch', funcname='xsntch', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='LineE', default=3.5, units='keV', softmin=0.0, softmax=20.0, hardmin=0.0, hardmax=20.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Width', default=1.0, units='keV', softmin=0.0, softmax=20.0, hardmin=0.0, hardmax=20.0, delta=0.02), XspecParameter(paramtype=ParamType.Default, name='CvrFract', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.005)]),
    'olivineabs': XspecModel(modeltype=ModelType.Mul, name='olivineabs', funcname='olivineabs', language=LanguageStyle.F77Style8, elo=0.01, ehi=1000000.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='moliv', default=1.0, units='10^-4', softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=100000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.01)]),
    'pcfabs': XspecModel(modeltype=ModelType.Mul, name='pcfabs', funcname='xsabsp', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='CvrFract', default=0.5, softmin=0.05, softmax=0.95, hardmin=0.0, hardmax=1.0, delta=0.01)]),
    'phabs': XspecModel(modeltype=ModelType.Mul, name='phabs', funcname='xsphab', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001)]),
    'plabs': XspecModel(modeltype=ModelType.Mul, name='plabs', funcname='xsplab', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='index', default=2.0, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='coef', default=1.0, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01)]),
    'polconst': XspecModel(modeltype=ModelType.Mul, name='polconst', funcname='polconst', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='A', default=1.0, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='psi', default=45.0, units='deg', softmin=-90.0, softmax=90.0, hardmin=-90.0, hardmax=90.0, delta=0.01)], can_cache=False),
    'pollin': XspecModel(modeltype=ModelType.Mul, name='pollin', funcname='pollin', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='A1', default=1.0, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Aslope', default=0.0, softmin=-5.0, softmax=5.0, hardmin=-5.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='psi1', default=45.0, units='deg', softmin=-90.0, softmax=90.0, hardmin=-90.0, hardmax=90.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='psislope', default=0.0, softmin=-5.0, softmax=5.0, hardmin=-5.0, hardmax=5.0, delta=0.01)], can_cache=False),
    'polpow': XspecModel(modeltype=ModelType.Mul, name='polpow', funcname='polpow', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Anorm', default=1.0, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Aindex', default=0.0, softmin=-5.0, softmax=5.0, hardmin=-5.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='psinorm', default=45.0, units='deg', softmin=-90.0, softmax=90.0, hardmin=-90.0, hardmax=90.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='psiindex', default=0.0, softmin=-5.0, softmax=5.0, hardmin=-5.0, hardmax=5.0, delta=0.01)], can_cache=False),
    'pwab': XspecModel(modeltype=ModelType.Mul, name='pwab', funcname='xspwab', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nHmin', default=1.0, units='10^22', softmin=1e-07, softmax=100000.0, hardmin=1e-07, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='nHmax', default=2.0, units='10^22', softmin=1e-07, softmax=100000.0, hardmin=1e-07, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='beta', default=1.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=20.0, delta=0.01)]),
    'redden': XspecModel(modeltype=ModelType.Mul, name='redden', funcname='xscred', language=LanguageStyle.F77Style4, elo=0.000372, ehi=0.00992, parameters=[XspecParameter(paramtype=ParamType.Default, name='E_BmV', default=0.05, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001)]),
    'smedge': XspecModel(modeltype=ModelType.Mul, name='smedge', funcname='xssmdg', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='edgeE', default=7.0, units='keV', softmin=0.1, softmax=100.0, hardmin=0.1, hardmax=100.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='MaxTau', default=1.0, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='index', default=-2.67, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='width', default=10.0, softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=100.0, delta=0.01)]),
    'spexpcut': XspecModel(modeltype=ModelType.Mul, name='spexpcut', funcname='superExpCutoff', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Ecut', default=10.0, units='keV', softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='alpha', default=1.0, softmin=-5.0, softmax=5.0, hardmin=-5.0, hardmax=5.0, delta=0.01)]),
    'spline': XspecModel(modeltype=ModelType.Mul, name='spline', funcname='xsspln', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Estart', default=0.1, units='keV', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Ystart', default=1.0, softmin=-1000000.0, softmax=1000000.0, hardmin=-1000000.0, hardmax=1000000.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Yend', default=1.0, softmin=-1000000.0, softmax=1000000.0, hardmin=-1000000.0, hardmax=1000000.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='YPstart', default=0.0, softmin=-1000000.0, softmax=1000000.0, hardmin=-1000000.0, hardmax=1000000.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='YPend', default=0.0, softmin=-1000000.0, softmax=1000000.0, hardmin=-1000000.0, hardmax=1000000.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Eend', default=15.0, units='keV', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.05)]),
    'SSS_ice': XspecModel(modeltype=ModelType.Mul, name='SSS_ice', funcname='xssssi', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='clumps', default=0.0, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.005)]),
    'swind1': XspecModel(modeltype=ModelType.Mul, name='swind1', funcname='swind1', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='column', default=6.0, softmin=3.0, softmax=50.0, hardmin=3.0, hardmax=50.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='log_xi', default=2.5, softmin=2.1, softmax=4.1, hardmin=2.1, hardmax=4.1, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='sigma', default=0.1, softmin=0.0, softmax=0.5, hardmin=0.0, hardmax=0.5, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'TBabs': XspecModel(modeltype=ModelType.Mul, name='TBabs', funcname='tbabs', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001)]),
    'TBfeo': XspecModel(modeltype=ModelType.Mul, name='TBfeo', funcname='tbfeo', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=-1e+38, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=-1e+38, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.01)]),
    'TBgas': XspecModel(modeltype=ModelType.Mul, name='TBgas', funcname='tbgas', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.01)]),
    'TBgrain': XspecModel(modeltype=ModelType.Mul, name='TBgrain', funcname='tbgrain', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='h2', default=0.2, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='rho', default=1.0, units='g/cm^3', frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='amin', default=0.025, units='mum', frozen=True, softmin=0.0, softmax=0.25, hardmin=0.0, hardmax=0.25, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='amax', default=0.25, units='mum', frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='PL', default=3.5, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01)]),
    'TBvarabs': XspecModel(modeltype=ModelType.Mul, name='TBvarabs', funcname='tbvabs', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H2', default=0.2, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='rho', default=1.0, units='g/cm^3', frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='amin', default=0.025, units='mum', frozen=True, softmin=0.0, softmax=0.25, hardmin=0.0, hardmax=0.25, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='amax', default=0.25, units='mum', frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='PL', default=3.5, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'TBpcf': XspecModel(modeltype=ModelType.Mul, name='TBpcf', funcname='tbpcf', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='pcf', default=0.5, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.01)]),
    'TBrel': XspecModel(modeltype=ModelType.Mul, name='TBrel', funcname='tbrel', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=0.0, units='10^22', softmin=-100000.0, softmax=100000.0, hardmin=-1000000.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=1e+38, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H2', default=0.2, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='rho', default=1.0, units='g/cm^3', frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='amin', default=0.025, units='mum', frozen=True, softmin=0.0, softmax=0.25, hardmin=0.0, hardmax=0.25, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='amax', default=0.25, units='mum', frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='PL', default=3.5, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='H_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C_dep', default=0.5, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O_dep', default=0.6, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na_dep', default=0.25, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg_dep', default=0.2, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al_dep', default=0.02, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si_dep', default=0.1, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S_dep', default=0.6, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl_dep', default=0.5, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar_dep', default=1.0, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca_dep', default=0.003, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr_dep', default=0.03, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_dep', default=0.3, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co_dep', default=0.05, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni_dep', default=0.04, frozen=True, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=-1.0, hardmax=10.0, delta=0.01)]),
    'uvred': XspecModel(modeltype=ModelType.Mul, name='uvred', funcname='xsred', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='E_BmV', default=0.05, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001)]),
    'varabs': XspecModel(modeltype=ModelType.Mul, name='varabs', funcname='xsabsv', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, units='sH22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, units='sHe22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, units='sC22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, units='sN22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, units='sO22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, units='sNe22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, units='sNa22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, units='sMg22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, units='sAl22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, units='sSi22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, units='sS22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, units='sCl22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, units='sAr22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, units='sCa22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, units='sCr22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, units='sFe22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, units='sCo22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, units='sNi22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01)]),
    'vphabs': XspecModel(modeltype=ModelType.Mul, name='vphabs', funcname='xsvphb', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01)]),
    'wabs': XspecModel(modeltype=ModelType.Mul, name='wabs', funcname='xsabsw', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001)]),
    'wndabs': XspecModel(modeltype=ModelType.Mul, name='wndabs', funcname='xswnab', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='WindowE', default=1.0, units='keV', softmin=0.05, softmax=20.0, hardmin=0.03, hardmax=20.0, delta=0.05)]),
    'xion': XspecModel(modeltype=ModelType.Mul, name='xion', funcname='xsxirf', language=LanguageStyle.F77Style4, elo=0.1, ehi=200.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='height', default=5.0, units='r_s', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=2.0), XspecParameter(paramtype=ParamType.Default, name='lxovrld', default=0.3, softmin=0.02, softmax=100.0, hardmin=0.02, hardmax=100.0, delta=0.02), XspecParameter(paramtype=ParamType.Default, name='rate', default=0.05, softmin=0.001, softmax=1.0, hardmin=0.001, hardmax=1.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='cosAng', default=0.9, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='inner', default=3.0, units='r_s', softmin=2.0, softmax=1000.0, hardmin=2.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='outer', default=100.0, units='r_s', softmin=2.1, softmax=100000.0, hardmin=2.1, hardmax=100000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='index', default=2.0, softmin=1.6, softmax=2.2, hardmin=1.6, hardmax=2.2, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Feabun', default=1.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='E_cut', default=150.0, units='keV', softmin=20.0, softmax=300.0, hardmin=20.0, hardmax=300.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Ref_type', default=1.0, frozen=True, softmin=1.0, softmax=3.0, hardmin=1.0, hardmax=3.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rel_smear', default=4.0, frozen=True, softmin=1.0, softmax=4.0, hardmin=1.0, hardmax=4.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Geometry', default=1.0, frozen=True, softmin=1.0, softmax=4.0, hardmin=1.0, hardmax=4.0, delta=0.01)]),
    'xscat': XspecModel(modeltype=ModelType.Mul, name='xscat', funcname='xscatmodel', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='NH', default=1.0, units='10^22', softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Xpos', default=0.5, softmin=0.0, softmax=0.99, hardmin=0.0, hardmax=0.999, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rext', default=10.0, units='arcsec', frozen=True, softmin=0.0, softmax=235.0, hardmin=0.0, hardmax=240.0, delta=1.0), XspecParameter(paramtype=ParamType.Switch, name='DustModel', default=1, frozen=True)]),
    'zbabs': XspecModel(modeltype=ModelType.Mul, name='zbabs', funcname='xszbabs', language=LanguageStyle.CStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=0.0001, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=1e-05), XspecParameter(paramtype=ParamType.Default, name='nHeI', default=1e-05, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=1e-06), XspecParameter(paramtype=ParamType.Default, name='nHeII', default=1e-06, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=1e-07), XspecParameter(paramtype=ParamType.Default, name='z', default=0.0, softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=1e-06)]),
    'zdust': XspecModel(modeltype=ModelType.Mul, name='zdust', funcname='mszdst', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Switch, name='method', default=1, frozen=True), XspecParameter(paramtype=ParamType.Default, name='E_BmV', default=0.1, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rv', default=3.1, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=0.0, softmax=20.0, hardmin=0.0, hardmax=20.0, delta=0.01)]),
    'zedge': XspecModel(modeltype=ModelType.Mul, name='zedge', funcname='xszedg', language=LanguageStyle.F77Style4, elo=1e-20, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='edgeE', default=7.0, units='keV', softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='MaxTau', default=1.0, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zhighect': XspecModel(modeltype=ModelType.Mul, name='zhighect', funcname='xszhcu', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='cutoffE', default=10.0, units='keV', softmin=0.01, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='foldE', default=15.0, units='keV', softmin=0.01, softmax=100.0, hardmin=0.0001, hardmax=200.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zigm': XspecModel(modeltype=ModelType.Mul, name='zigm', funcname='zigm', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Scale, name='redshift', default=0.0, frozen=True), XspecParameter(paramtype=ParamType.Switch, name='model', default=0, frozen=True), XspecParameter(paramtype=ParamType.Switch, name='lyman_limit', default=1, frozen=True)]),
    'zpcfabs': XspecModel(modeltype=ModelType.Mul, name='zpcfabs', funcname='xszabp', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='CvrFract', default=0.5, softmin=0.05, softmax=0.95, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zphabs': XspecModel(modeltype=ModelType.Mul, name='zphabs', funcname='xszphb', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zxipab': XspecModel(modeltype=ModelType.Mul, name='zxipab', funcname='zxipab', language=LanguageStyle.F77Style4, elo=0.01, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nHmin', default=0.01, units='10^22', softmin=1e-07, softmax=1000.0, hardmin=1e-07, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='nHmax', default=10.0, units='10^22', softmin=1e-07, softmax=1000.0, hardmin=1e-07, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='beta', default=0.0, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='log_xi', default=3.0, softmin=-3.0, softmax=6.0, hardmin=-3.0, hardmax=6.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01)]),
    'zxipcf': XspecModel(modeltype=ModelType.Mul, name='zxipcf', funcname='zxipcf', language=LanguageStyle.CppStyle8, elo=0.01, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Nh', default=10.0, units='10^22', softmin=0.05, softmax=500.0, hardmin=0.05, hardmax=500.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='log_xi', default=3.0, softmin=-3.0, softmax=6.0, hardmin=-3.0, hardmax=6.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='CvrFract', default=0.5, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zredden': XspecModel(modeltype=ModelType.Mul, name='zredden', funcname='xszcrd', language=LanguageStyle.F77Style4, elo=0.000372, ehi=0.00992, parameters=[XspecParameter(paramtype=ParamType.Default, name='E_BmV', default=0.05, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zsmdust': XspecModel(modeltype=ModelType.Mul, name='zsmdust', funcname='msldst', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='E_BmV', default=0.1, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='ExtIndex', default=1.0, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rv', default=3.1, frozen=True, softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, units='z', frozen=True, softmin=0.0, softmax=20.0, hardmin=0.0, hardmax=20.0, delta=0.01)]),
    'zTBabs': XspecModel(modeltype=ModelType.Mul, name='zTBabs', funcname='ztbabs', language=LanguageStyle.CppStyle8, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zvarabs': XspecModel(modeltype=ModelType.Mul, name='zvarabs', funcname='xszvab', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='H', default=1.0, units='sH22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, units='sHe22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, units='sC22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, units='sN22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, units='sO22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, units='sNe22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, units='sNa22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, units='sMg22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, units='sAl22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, units='sSi22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, units='sS22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, units='sCl22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, units='sAr22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, units='sCa22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, units='sCr22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, units='sFe22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, units='sCo22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, units='sNi22', frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=10000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zvfeabs': XspecModel(modeltype=ModelType.Mul, name='zvfeabs', funcname='xszvfe', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='metals', default=1.0, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='FEabun', default=1.0, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='FEKedge', default=7.11, units='keV', softmin=7.0, softmax=9.5, hardmin=7.0, hardmax=9.5, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zvphabs': XspecModel(modeltype=ModelType.Mul, name='zvphabs', funcname='xszvph', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='He', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='C', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='N', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='O', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ne', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Na', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mg', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Al', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Si', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='S', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cl', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ar', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ca', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Cr', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Co', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Ni', default=1.0, frozen=True, softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zwabs': XspecModel(modeltype=ModelType.Mul, name='zwabs', funcname='xszabs', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=100000.0, hardmin=0.0, hardmax=1000000.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zwndabs': XspecModel(modeltype=ModelType.Mul, name='zwndabs', funcname='xszwnb', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='nH', default=1.0, units='10^22', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='WindowE', default=1.0, units='keV', softmin=0.05, softmax=20.0, hardmin=0.03, hardmax=20.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'cflux': XspecModel(modeltype=ModelType.Con, name='cflux', funcname='cflux', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Emin', default=0.5, units='keV', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Emax', default=10.0, units='keV', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='lg10Flux', default=-12.0, units='cgs', softmin=-100.0, softmax=100.0, hardmin=-100.0, hardmax=100.0, delta=0.01)]),
    'clumin': XspecModel(modeltype=ModelType.Con, name='clumin', funcname='clumin', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Emin', default=0.5, units='keV', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Emax', default=10.0, units='keV', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='lg10Lum', default=40.0, units='cgs', softmin=-100.0, softmax=100.0, hardmin=-100.0, hardmax=100.0, delta=0.01)]),
    'cglumin': XspecModel(modeltype=ModelType.Con, name='cglumin', funcname='cglumin', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Emin', default=0.5, units='keV', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Emax', default=10.0, units='keV', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Distance', default=10.0, units='kpc', frozen=True, softmin=0.0, softmax=10000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='lg10Lum', default=40.0, units='cgs', softmin=-100.0, softmax=100.0, hardmin=-100.0, hardmax=100.0, delta=0.01)]),
    'cpflux': XspecModel(modeltype=ModelType.Con, name='cpflux', funcname='cpflux', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Emin', default=0.5, units='keV', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Emax', default=10.0, units='keV', frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Flux', default=1.0, softmin=0.0, softmax=10000000000.0, hardmin=0.0, hardmax=10000000000.0, delta=0.01)]),
    'gsmooth': XspecModel(modeltype=ModelType.Con, name='gsmooth', funcname='gsmooth', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Sig_6keV', default=1.0, units='keV', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Index', default=0.0, frozen=True, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01)]),
    'ireflect': XspecModel(modeltype=ModelType.Con, name='ireflect', funcname='ireflct', language=LanguageStyle.CppStyle8, elo=0.001, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=0.0, softmin=-1.0, softmax=1000000.0, hardmin=-1.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.45, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='T_disk', default=30000.0, units='K', frozen=True, softmin=10000.0, softmax=1000000.0, hardmin=10000.0, hardmax=1000000.0, delta=1000.0), XspecParameter(paramtype=ParamType.Default, name='xi', default=1.0, units='erg cm/s', softmin=0.0, softmax=1000.0, hardmin=0.0, hardmax=5000.0, delta=0.1)]),
    'kdblur': XspecModel(modeltype=ModelType.Con, name='kdblur', funcname='kdblur', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Index', default=3.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rin_G', default=4.5, frozen=True, softmin=1.235, softmax=400.0, hardmin=1.235, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout_G', default=100.0, frozen=True, softmin=1.235, softmax=400.0, hardmin=1.235, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Incl', default=30.0, units='deg', softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=1.0)]),
    'kdblur2': XspecModel(modeltype=ModelType.Con, name='kdblur2', funcname='kdblur2', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Index', default=3.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rin_G', default=4.5, frozen=True, softmin=1.235, softmax=400.0, hardmin=1.235, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout_G', default=100.0, frozen=True, softmin=1.235, softmax=400.0, hardmin=1.235, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Incl', default=30.0, units='deg', softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Rbreak', default=20.0, frozen=True, softmin=1.235, softmax=400.0, hardmin=1.235, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Index1', default=3.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1)]),
    'kerrconv': XspecModel(modeltype=ModelType.Con, name='kerrconv', funcname='spinconv', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Index1', default=3.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Index2', default=3.0, frozen=True, softmin=-10.0, softmax=10.0, hardmin=-10.0, hardmax=10.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='r_br_g', default=6.0, frozen=True, softmin=1.0, softmax=400.0, hardmin=1.0, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='a', default=0.998, softmin=0.0, softmax=0.998, hardmin=0.0, hardmax=0.998, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Incl', default=30.0, units='deg', frozen=True, softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Rin_ms', default=1.0, frozen=True, softmin=1.0, softmax=400.0, hardmin=1.0, hardmax=400.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout_ms', default=400.0, frozen=True, softmin=1.0, softmax=400.0, hardmin=1.0, hardmax=400.0, delta=0.1)]),
    'lsmooth': XspecModel(modeltype=ModelType.Con, name='lsmooth', funcname='lsmooth', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Sig_6keV', default=1.0, units='keV', softmin=0.0, softmax=10.0, hardmin=0.0, hardmax=20.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='Index', default=0.0, frozen=True, softmin=-1.0, softmax=1.0, hardmin=-1.0, hardmax=1.0, delta=0.01)]),
    'partcov': XspecModel(modeltype=ModelType.Con, name='partcov', funcname='PartialCovering', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='CvrFract', default=0.5, softmin=0.05, softmax=0.95, hardmin=0.0, hardmax=1.0, delta=0.01)]),
    'rdblur': XspecModel(modeltype=ModelType.Con, name='rdblur', funcname='rdblur', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Betor10', default=-2.0, frozen=True, softmin=-10.0, softmax=20.0, hardmin=-10.0, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Rin_M', default=10.0, frozen=True, softmin=6.0, softmax=1000.0, hardmin=6.0, hardmax=10000.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='Rout_M', default=1000.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=10000000.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Incl', default=30.0, units='deg', softmin=0.0, softmax=90.0, hardmin=0.0, hardmax=90.0, delta=0.05)]),
    'reflect': XspecModel(modeltype=ModelType.Con, name='reflect', funcname='reflct', language=LanguageStyle.CppStyle8, elo=0.001, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=0.0, softmin=-1.0, softmax=1000000.0, hardmin=-1.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.0, softmax=1000000.0, hardmin=0.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.45, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.01)]),
    'rfxconv': XspecModel(modeltype=ModelType.Con, name='rfxconv', funcname='rfxconv', language=LanguageStyle.CppStyle8, elo=0.001, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=-1.0, softmin=-1.0, softmax=1000000.0, hardmin=-1.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=4.0, hardmin=0.0, hardmax=4.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.5, softmax=3.0, hardmin=0.5, hardmax=3.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.5, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='log_xi', default=1.0, softmin=1.0, softmax=6.0, hardmin=1.0, hardmax=6.0, delta=0.02)]),
    'rgsxsrc': XspecModel(modeltype=ModelType.Con, name='rgsxsrc', funcname='rgsxsrc', language=LanguageStyle.F77Style4, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='order', default=-1.0, frozen=True, softmin=-3.0, softmax=-1.0, hardmin=-3.0, hardmax=-1.0, delta=1.0)]),
    'simpl': XspecModel(modeltype=ModelType.Con, name='simpl', funcname='simpl', language=LanguageStyle.CppStyle8, elo=0.05, ehi=1000000.0, parameters=[XspecParameter(paramtype=ParamType.Default, name='Gamma', default=2.3, softmin=1.1, softmax=4.0, hardmin=1.0, hardmax=5.0, delta=0.05), XspecParameter(paramtype=ParamType.Default, name='FracSctr', default=0.05, softmin=0.0, softmax=0.4, hardmin=0.0, hardmax=1.0, delta=0.005), XspecParameter(paramtype=ParamType.Default, name='UpScOnly', default=1.0, frozen=True, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=1.0)]),
    'thcomp': XspecModel(modeltype=ModelType.Con, name='thcomp', funcname='thcompf', language=LanguageStyle.F77Style4, elo=0.03, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Gamma_tau', default=1.7, softmin=1.001, softmax=5.0, hardmin=1.001, hardmax=10.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='kT_e', default=50.0, units='keV', softmin=0.5, softmax=150.0, hardmin=0.5, hardmax=150.0, delta=0.1), XspecParameter(paramtype=ParamType.Default, name='cov_frac', default=1.0, softmin=0.0, softmax=1.0, hardmin=0.0, hardmax=1.0, delta=0.001), XspecParameter(paramtype=ParamType.Default, name='z', default=0.0, frozen=True, softmin=0.0, softmax=5.0, hardmin=0.0, hardmax=5.0, delta=0.01)]),
    'vashift': XspecModel(modeltype=ModelType.Con, name='vashift', funcname='vashift', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=-10000.0, softmax=10000.0, hardmin=-10000.0, hardmax=10000.0, delta=10.0)]),
    'vmshift': XspecModel(modeltype=ModelType.Con, name='vmshift', funcname='vmshift', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Velocity', default=0.0, units='km/s', frozen=True, softmin=-10000.0, softmax=10000.0, hardmin=-10000.0, hardmax=10000.0, delta=10.0)]),
    'xilconv': XspecModel(modeltype=ModelType.Con, name='xilconv', funcname='xilconv', language=LanguageStyle.CppStyle8, elo=0.001, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='rel_refl', default=-1.0, softmin=-1.0, softmax=1000000.0, hardmin=-1.0, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='redshift', default=0.0, frozen=True, softmin=0.0, softmax=4.0, hardmin=0.0, hardmax=4.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Fe_abund', default=1.0, frozen=True, softmin=0.5, softmax=3.0, hardmin=0.5, hardmax=3.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='cosIncl', default=0.5, frozen=True, softmin=0.05, softmax=0.95, hardmin=0.05, hardmax=0.95, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='log_xi', default=1.0, softmin=1.0, softmax=6.0, hardmin=1.0, hardmax=6.0, delta=0.02), XspecParameter(paramtype=ParamType.Default, name='cutoff', default=300.0, units='keV', frozen=True, softmin=20.0, softmax=300.0, hardmin=20.0, hardmax=300.0, delta=1.0)]),
    'zashift': XspecModel(modeltype=ModelType.Con, name='zashift', funcname='zashift', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'zmshift': XspecModel(modeltype=ModelType.Con, name='zmshift', funcname='zmshift', language=LanguageStyle.CppStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Redshift', default=0.0, frozen=True, softmin=-0.999, softmax=10.0, hardmin=-0.999, hardmax=10.0, delta=0.01)]),
    'bwcycl': XspecModel(modeltype=ModelType.Add, name='bwcycl', funcname='beckerwolff', language=LanguageStyle.CStyle8, elo=0.0, ehi=1e+20, parameters=[XspecParameter(paramtype=ParamType.Default, name='Radius', default=10.0, units='km', frozen=True, softmin=5.0, softmax=20.0, hardmin=5.0, hardmax=20.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='Mass', default=1.4, units='Solar', frozen=True, softmin=1.0, softmax=3.0, hardmin=1.0, hardmax=3.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='csi', default=3.0, softmin=0.01, softmax=20.0, hardmin=0.01, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='del', default=1.0, softmin=0.01, softmax=20.0, hardmin=0.01, hardmax=20.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='B', default=4.0, units='1e12G', softmin=0.01, softmax=100.0, hardmin=0.01, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Mdot', default=1.0, units='1e17g/s', softmin=1e-06, softmax=1000000.0, hardmin=1e-06, hardmax=1000000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='Te', default=5.0, units='keV', softmin=1.3, softmax=100.0, hardmin=1.3, hardmax=100.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='r0', default=44.0, units='m', softmin=10.0, softmax=1000.0, hardmin=10.0, hardmax=1000.0, delta=0.01), XspecParameter(paramtype=ParamType.Default, name='D', default=5.0, units='kpc', frozen=True, softmin=1.0, softmax=20.0, hardmin=1.0, hardmax=20.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='BBnorm', default=0.0, frozen=True, softmin=0.0, softmax=100.0, hardmin=0.0, hardmax=100.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='CYCnorm', default=1.0, frozen=True, softmin=-1.0, softmax=100.0, hardmin=-1.0, hardmax=100.0, delta=1.0), XspecParameter(paramtype=ParamType.Default, name='FFnorm', default=1.0, frozen=True, softmin=-1.0, softmax=100.0, hardmin=-1.0, hardmax=100.0, delta=1.0)])
}


def info(model: str) -> XspecModel:
    """Return information on the Xspec model from the model.dat file.

    This returns the information on the model as taken from the Xspec
    model library used to build this model.

    Parameters
    ----------
    name : str
        The Xspec model name (case-insensitive).

    Returns
    -------
    model : XspecModel
        The dataclass that describes the mode;.

    See Also
    --------
    list_models

    Examples
    --------

    >>> m = info('apec')
    >>> m.name
    'apec'
    >>> m.modeltype
    <ModelType.Add: 1>
    >>> [(p.name, p.default, p.units) for p in m.parameters]
    [('kT', 1.0, 'keV'), ('Abundanc', 1.0, None), ('Redshift', 0.0, None)]

    """
    # We want case-insensitive comparison but for the keys to retain
    # their case. Using casefold() rather than lower() is a bit OTT
    # here as I would bet model.dat is meant to be US-ASCII.
    model = str(model)
    check = model.casefold()
    out = next((v for k, v in _info.items() if k.casefold() == check), None)
    if out is None:
        raise ValueError(f"Unrecognized Xspec model '{model}'")

    return out


# Do we need Optional here?
def list_models(modeltype: ModelType | None = None,
                language: LanguageStyle | None = None) -> list[str]:
    """Returns the names of Xspec models from the model.dat file.

    This returns the information on the model as taken from the Xspec
    model library used to build this model.

    Parameters
    ----------
    modeltype : ModelType or None, optional
        If not None then restrict the list to this model type.
    language : LanguageStyle or None, optional
        If not None then restrict the list to this language type.

    Returns
    -------
    models : list of str
        A sorted list of the model names.

    See Also
    --------
    info

    Notes
    -----
    The restrictions are combined, so setting both `modeltype` and
    `language` will select only those models which match both filters.

    Examples
    --------

    >>> len(list_models())
    231

    >>> 'tbabs' in list_models()
    False
    >>> 'TBabs' in list_models()
    True

    With Xspec 12.13.1 / HEASOFT 6.29:

    >>> list_models(modeltype=ModelType.Con)
    ['cflux', 'clumin', 'cpflux', 'gsmooth', ..., 'zashift', 'zmshift']

    >>> list_models(modeltype=ModelType.Con, language=LanguageStyle.F77Style4)
    ['kyconv', 'rgsxsrc', 'thcomp']

    """

    out = set()
    for k, v in _info.items():

        if modeltype is not None and v.modeltype != modeltype:
            continue

        if language is not None and v.language != language:
            continue

        out.add(k)

    return sorted(out)


if __INITIALIZED__:
    from .primitive import get_primitive
